from ase import Atoms
from gpaw import GPAW, FermiDirac
from gpaw.mixer import MixerSum
from gpaw.test import equal

a = 2.87
bulk = Atoms('Fe2',
             scaled_positions=[(0, 0, 0), (0.5, 0.5, 0.5)],
             magmoms=[2.20, 2.20],
             cell=(a, a, a),
             pbc=True)
mom0 = sum(bulk.get_initial_magnetic_moments())
h = 0.2
conv = {'eigenstates': 0.1, 'density': 0.1, 'energy': 0.01}
calc = GPAW(h=h,
            eigensolver='rmm-diis',
            mixer=MixerSum(0.1, 3),
            nbands=11,
            kpts=(3, 3, 3),
            convergence=conv,
            occupations=FermiDirac(0.1, fixmagmom=True))
bulk.set_calculator(calc)
bulk.get_potential_energy()
mom = calc.get_magnetic_moment()
equal(mom, mom0, 1e-5)



from __future__ import print_function

from gpaw.atom.all_electron import AllElectron
from gpaw.test import equal

data = []
def out(a,b,c,d,e,f):
    data.append( (a,b,c,d,e,f) )

ETotal = {'Be': -14.572+0.012, 'Ne': -128.548 -0.029, 'Mg': -199.612 - 0.005 }
EX = {'Be': -2.666 - 0.010, 'Ne': -12.107 -0.122, 'Mg': -15.992 -0.092 }
EHOMO = {'Be': -0.309 + 0.008, 'Ne': -0.851 + 0.098, 'Mg': -0.253 + 0.006}
eignum = {'Be': 0, 'Ne':3, 'Mg':0 }

for xcname in ['GLLB','GLLBSC']:
    atoms = ['Be','Ne','Mg']
    for atom in atoms:
        # Test AllElectron GLLB
        GLLB = AllElectron(atom, xcname = xcname, scalarrel = False, gpernode = 600)
        GLLB.run()
        
        out("Total energy", xcname+"1D", atom,  ETotal[atom] , GLLB.ETotal,"Ha")
        out("Exchange energy", xcname+"1D", atom, EX[atom], GLLB.Exc,"Ha")
        out("HOMO Eigenvalue", xcname+"1D", atom, EHOMO[atom], GLLB.e_j[-1],"Ha")
        if xcname == 'GLLB':
            equal(GLLB.ETotal, ETotal[atom], tolerance=1e-2)
            equal(GLLB.Exc, EX[atom], tolerance=1e-2)
            equal(GLLB.e_j[-1], EHOMO[atom], tolerance=1e-2)
            
    print("             Quanity        Method    Symbol     Ref[1]         GPAW      Unit  ")
    for a,b,c,d,e,f in data:
        print("%20s %10s %10s   %10.3f   %10.3f   %5s" % (a,b,c,d,e,f))
    
print("""References:
[1] Self-consistent approximation to the Kohn-Sham exchange potential
Gritsenko, Oleg; Leeuwen, Robert van; Lenthe, Erik van; Baerends, Evert Jan
Phys. Rev. A Vol. 51 p. 1944""")




from gpaw.fdtd.poisson_fdtd import QSFDTD
from gpaw.fdtd.polarizable_material import PermittivityPlus, PolarizableMaterial, \
    PolarizableSphere, PolarizableBox, \
    PolarizableEllipsoid, PolarizableRod, \
    PolarizableTetrahedron
from gpaw.test import equal

# Whole simulation cell (Angstroms)
cell = [40, 40, 20]

# Classical subsystem
classical_material = PolarizableMaterial()
classical_material.add_component(PolarizableSphere(permittivity = PermittivityPlus(data = [[1.20, 0.20, 25.0]]),
                                                   center = [10, 10, 10],
                                                   radius = 4.5))
classical_material.add_component(PolarizableBox(permittivity = PermittivityPlus(data = [[1.40, 0.20, 25.0]]),
                                                corner1 = [18.1, 5.1, 5.1],
                                                corner2 = [22.9, 14.9, 14.9]))
classical_material.add_component(PolarizableEllipsoid(permittivity = PermittivityPlus(data = [[1.60, 0.20, 25.0]]),
                                                center  = [30.0, 10.0, 10.0],
                                                radii   = [ 3.9, 5.9, 4.9]))
classical_material.add_component(PolarizableRod(permittivity = PermittivityPlus(data = [[1.80, 0.20, 25.0]]),
                                                corners = [[10.0, 21.5, 10.0], [10.0, 33.5, 10.0]],
                                                round_corners = True,
                                                radius = 3.9))
classical_material.add_component(PolarizableRod(permittivity = PermittivityPlus(data = [[1.00, 0.20, 25.0]]),
                                                corners = [[20.0, 21.5, 10.0], [25.0, 33.5, 10.0]],
                                                round_corners = False,
                                                radius = 2.9))
classical_material.add_component(PolarizableTetrahedron(permittivity = PermittivityPlus(data = [[0.80, 0.20, 25.0]]),
                                                        corners = [[24.1, 16.1, 5.1],
                                                                   [30.1, 36.1, 6.1],
                                                                   [36.4, 27.6, 7.1],
                                                                   [30.0, 25.0, 14.9]]))


# Wrap calculators
qsfdtd = QSFDTD(classical_material = classical_material,
                atoms              = None,
                cells              = (cell, 2.00),
                spacings           = [1.60, 0.40],
                remove_moments     = (1, 1))

# Run
energy = qsfdtd.ground_state('gs.gpw', eigensolver = 'cg', nbands = -1)
qsfdtd.time_propagation('gs.gpw', kick_strength=[0.000, 0.000, 0.001], time_step=10, iterations=5, dipole_moment_file='dmCl.dat')

# Restart and run
qsfdtd.write('td.gpw', mode='all')
qsfdtd.time_propagation('td.gpw', kick_strength=None, time_step=10, iterations=5, dipole_moment_file='dmCl.dat')

# Test
ref_cl_dipole_moment = [ -1.01218549e-04,  -3.03603883e-05,   1.86063875e-01]
tol = 1e-6
equal(qsfdtd.td_calc.hamiltonian.poisson.get_classical_dipole_moment(), ref_cl_dipole_moment, tol)




class Contribution:
    def __init__(self, nlfunc, weight):
        self.weight = weight
        self.nlfunc = nlfunc
        nlfunc.add_contribution(self)

    def initialize(self):
        raise NotImplementedError

    def initialize_1d(self):
        raise NotImplementedError

    def get_name(self):
        raise NotImplementedError

    def set_positions(self, spos_ac):
        pass

    def get_desc(self):
        raise NotImplementedError

    def calculate_spinpaired(self, e_g, n_g, v_g):
        raise NotImplementedError

    def calculate_spinpolarized(self, e_g, na_g, va_g, nb_g, vb_g):
        raise NotImplementedError

    def calculate_energy_and_derivatives(self, setup, D_sp, H_sp):
        raise NotImplementedError
    
    def add_smooth_xc_potential_and_energy_1d(self, vt_g):
        raise NotImplementedError

    def add_extra_setup_date(self, dict):
        raise NotImplementedError
        



from ase.build import molecule
from gpaw.mpi import world
from gpaw import GPAW, Mixer

size = (world.size // 2, 2) if world.size > 1 else (1, 1)

energies = []
for use_elpa in [1, 0]:
    atoms = molecule('CH3CH2OH', vacuum=2.5)
    #atoms = molecule('H2', vacuum=3.0)
    calc = GPAW(mode='lcao', basis='dzp',
                h=0.25,
                parallel=dict(sl_default=(size[0], size[1], 3),
                              use_elpa=use_elpa),
                mixer=Mixer(0.5, 5, 50.0))
    atoms.calc = calc
    E = atoms.get_potential_energy()
    energies.append(E)
err = abs(energies[1] - energies[0])
assert err < 1e-10, err



from ase import Atoms
from gpaw import GPAW, restart

from gpaw.test import equal
a = 6.0
atom = Atoms('H', magmoms=[1.0], cell=(a, a, a))
molecule = Atoms('H2', positions=[(0, 0, 0), (0, 0, 0.737)], cell=(a, a, a))
atom.center()
molecule.center()

calc = GPAW(xc='LDA-PZ-SIC',
            eigensolver='rmm-diis',
            txt='h2.sic.txt',
            setups='hgh')

atom.set_calculator(calc)
e1 = atom.get_potential_energy()

molecule.set_calculator(calc)
e2 = molecule.get_potential_energy()
F_ac = molecule.get_forces()
de = 2 * e1 - e2
# equal(de, 4.5, 0.1)

# Test forces ...

calc.write('H2.gpw', mode='all')
atoms, calc = restart('H2.gpw')
e2b = atoms.get_potential_energy()
equal(e2, e2b, 0.0001)



# Copyright (C) 2003  CAMP
# Please see the accompanying LICENSE file for further information.
from __future__ import print_function
import hashlib
import os
import re
import xml.sax
from glob import glob
from math import sqrt, pi
from distutils.version import LooseVersion

import numpy as np
from ase.data import atomic_names
from ase.units import Bohr, Hartree

from gpaw import setup_paths, extra_parameters
from gpaw.xc.pawcorrection import PAWXCCorrection
from gpaw.mpi import broadcast
from gpaw.atom.radialgd import (AERadialGridDescriptor,
                                AbinitRadialGridDescriptor)
from gpaw.atom.shapefunc import shape_functions

try:
    import gzip
except ImportError:
    has_gzip = False
else:
    has_gzip = True


class SetupData:
    """Container class for persistent setup attributes and XML I/O."""
    def __init__(self, symbol, xcsetupname, name='paw', readxml=True,
                 zero_reference=False, world=None,
                 generator_version=None):
        self.symbol = symbol
        self.setupname = xcsetupname
        self.name = name
        self.zero_reference = zero_reference
        self.generator_version = generator_version

        # Default filename if this setup is written
        if name is None or name == 'paw':
            self.stdfilename = '%s.%s' % (symbol, self.setupname)
        else:
            self.stdfilename = '%s.%s.%s' % (symbol, name, self.setupname)

        self.filename = None  # full path if this setup was loaded from file
        self.fingerprint = None  # hash value of file data if applicable

        self.Z = None
        self.Nc = None
        self.Nv = None

        # Quantum numbers, energies
        self.n_j = []
        self.l_j = []
        self.l_orb_j = self.l_j  # pointer to same list!
        self.f_j = []
        self.eps_j = []
        self.e_kin_jj = None  # <phi | T | phi> - <phit | T | phit>

        self.rgd = None

        # Parameters for compensation charge expansion functions:
        self.shape_function = {'type': 'undefined', 'rc': np.nan}

        # State identifier, like "X-2s" or "X-p1", where X is chemical symbol,
        # for bound and unbound states
        self.id_j = []

        # Partial waves, projectors
        self.phi_jg = []
        self.phit_jg = []
        self.pt_jg = []
        self.rcut_j = []

        # Densities, potentials
        self.nc_g = None
        self.nct_g = None
        self.nvt_g = None
        self.vbar_g = None

        # Kinetic energy densities of core electrons
        self.tauc_g = None
        self.tauct_g = None

        # Reference energies
        self.e_kinetic = 0.0
        self.e_xc = 0.0
        self.e_electrostatic = 0.0
        self.e_total = 0.0
        self.e_kinetic_core = 0.0

        # Generator may store description of setup in this string
        self.generatorattrs = []
        self.generatordata = ''

        # Optional quantities, normally not used
        self.X_p = None
        self.X_pg = None
        self.ExxC = None
        self.X_gamma = None
        self.extra_xc_data = {}
        self.phicorehole_g = None
        self.fcorehole = 0.0
        self.lcorehole = None
        self.ncorehole = None
        self.core_hole_e = None
        self.core_hole_e_kin = None
        self.has_corehole = False

        # Parameters for zero-potential:
        self.l0 = None
        self.e0 = None
        self.r0 = None
        self.nderiv0 = None

        self.orbital_free = False  # orbital-free DFT

        if readxml:
            self.read_xml(world=world)

    def __repr__(self):
        return ('{0}({symbol!r}, {setupname!r}, name={name!r}, '
                'generator_version={generator_version!r}, ...)'
                .format(self.__class__.__name__, **vars(self)))

    def append(self, n, l, f, e, rcut, phi_g, phit_g, pt_g):
        self.n_j.append(n)
        self.l_j.append(l)
        self.f_j.append(f)
        self.eps_j.append(e)
        self.rcut_j.append(rcut)
        self.phi_jg.append(phi_g)
        self.phit_jg.append(phit_g)
        self.pt_jg.append(pt_g)

    def read_xml(self, source=None, world=None):
        PAWXMLParser(self).parse(source=source, world=world)
        nj = len(self.l_j)
        self.e_kin_jj.shape = (nj, nj)

    def is_compatible(self, xc):
        return xc.get_setup_name() == self.setupname

    def print_info(self, text, setup):
        if self.phicorehole_g is None:
            text(self.symbol + '-setup:')
        else:
            text('%s-setup (%.1f core hole):' % (self.symbol, self.fcorehole))
        text('  name:', atomic_names[self.Z])
        text('  id:', self.fingerprint)
        text('  Z:', self.Z)
        text('  valence:', self.Nv)
        if self.phicorehole_g is None:
            text('  core: %d' % self.Nc)
        else:
            text('  core: %.1f' % self.Nc)
        text('  charge:', self.Z - self.Nv - self.Nc)
        if setup.HubU is not None:
            text('  Hubbard U: %f eV (l=%d, scale=%s)' %
                 (setup.HubU * Hartree, setup.Hubl, bool(setup.Hubs)))
        text('  file:', self.filename)
        text('  compensation charges: {}, rc={:.2f}, lmax={}'
             .format(self.shape_function['type'],
                     self.shape_function['rc'] * Bohr,
                     setup.lmax))
        text(('  cutoffs: {:.2f}(filt), {:.2f}(core),'
              .format(setup.rcutfilter * Bohr,
                      setup.rcore * Bohr)))
        text('  valence states:')
        text('                energy  radius')
        j = 0
        for n, l, f, eps in zip(self.n_j, self.l_j, self.f_j, self.eps_j):
            if n > 0:
                f = '(%.2f)' % f
                text('    %d%s%-5s %9.3f   %5.3f' % (
                    n, 'spdf'[l], f, eps * Hartree, self.rcut_j[j] * Bohr))
            else:
                text('    *%s       %9.3f   %5.3f' % (
                    'spdf'[l], eps * Hartree, self.rcut_j[j] * Bohr))
            j += 1
        text()

    def create_compensation_charge_functions(self, lmax):
        """Create shape functions used to expand compensation charges."""
        g_lg = shape_functions(self.rgd, **self.shape_function, lmax=lmax)
        return g_lg

    def get_smooth_core_density_integral(self, Delta0):
        return -Delta0 * sqrt(4 * pi) - self.Z + self.Nc

    def get_overlap_correction(self, Delta0_ii):
        return sqrt(4.0 * pi) * Delta0_ii

    def get_linear_kinetic_correction(self, T0_qp):
        e_kin_jj = self.e_kin_jj
        nj = len(e_kin_jj)
        K_q = []
        for j1 in range(nj):
            for j2 in range(j1, nj):
                K_q.append(e_kin_jj[j1, j2])
        K_p = sqrt(4 * pi) * np.dot(K_q, T0_qp)
        return K_p

    def find_core_density_cutoff(self, nc_g):
        if self.Nc == 0:
            return 1.0
        else:
            rgd = self.rgd
            N = 0.0
            g = self.rgd.N - 1
            while N < 1e-7:
                N += sqrt(4 * pi) * nc_g[g] * rgd.r_g[g]**2 * rgd.dr_g[g]
                g -= 1
            return rgd.r_g[g]

    def get_xc_correction(self, rgd, xc, gcut2, lcut):
        phicorehole_g = self.phicorehole_g
        if phicorehole_g is not None:
            phicorehole_g = phicorehole_g[:gcut2].copy()

        xc_correction = PAWXCCorrection(
            [phi_g[:gcut2] for phi_g in self.phi_jg],
            [phit_g[:gcut2] for phit_g in self.phit_jg],
            self.nc_g[:gcut2] / sqrt(4 * pi),
            self.nct_g[:gcut2] / sqrt(4 * pi),
            rgd,
            list(enumerate(self.l_j)),
            min(2 * lcut, 4),
            self.e_xc,
            phicorehole_g,
            self.fcorehole,
            None if self.tauc_g is None else self.tauc_g[:gcut2].copy(),
            None if self.tauct_g is None else self.tauct_g[:gcut2].copy())

        return xc_correction

    def write_xml(self):
        l_j = self.l_j
        xml = open(self.stdfilename, 'w')

        print('<?xml version="1.0"?>', file=xml)
        print('<paw_dataset version="{version}">'
              .format(version=self.version),
              file=xml)
        name = atomic_names[self.Z].title()
        comment1 = name + ' setup for the Projector Augmented Wave method.'
        comment2 = 'Units: Hartree and Bohr radii.'
        comment2 += ' ' * (len(comment1) - len(comment2))
        print('  <!--', comment1, '-->', file=xml)
        print('  <!--', comment2, '-->', file=xml)

        print(('  <atom symbol="%s" Z="%d" core="%r" valence="%d"/>' %
               (self.symbol, self.Z, self.Nc, self.Nv)), file=xml)
        if self.orbital_free:
            type = 'OFDFT'
            name = self.setupname
        elif self.setupname == 'LDA':
            type = 'LDA'
            name = 'PW'
        else:
            type = 'GGA'
            name = self.setupname
        print('  <xc_functional type="%s" name="%s"/>' % (type, name),
              file=xml)
        gen_attrs = ' '.join(['%s="%s"' % (key, value) for key, value
                              in self.generatorattrs])
        print('  <generator %s>' % gen_attrs, file=xml)
        print('    %s' % self.generatordata, file=xml)
        print('  </generator>', file=xml)
        print('  <ae_energy kinetic="%r" xc="%r"' %
              (self.e_kinetic, self.e_xc), file=xml)
        print('             electrostatic="%r" total="%r"/>' %
              (self.e_electrostatic, self.e_total), file=xml)

        print('  <core_energy kinetic="%r"/>' % self.e_kinetic_core, file=xml)
        print('  <valence_states>', file=xml)
        line1 = '    <state n="%d" l="%d" f="%r" rc="%r" e="%r" id="%s"/>'
        line2 = '    <state       l="%d"        rc="%r" e="%r" id="%s"/>'

        for id, l, n, f, e, rc in zip(self.id_j, l_j, self.n_j, self.f_j,
                                      self.eps_j, self.rcut_j):
            if n > 0:
                print(line1 % (n, l, f, rc, e, id), file=xml)
            else:
                print(line2 % (l, rc, e, id), file=xml)
        print('  </valence_states>', file=xml)

        print(self.rgd.xml('g1'), file=xml)

        print('  <shape_function type="{type}" rc="{rc}"/>'
              .format(**self.shape_function), file=xml)

        if self.r0 is None:
            # Old setups:
            xml.write('  <zero_potential grid="g1">\n')
        elif self.l0 is None:
            xml.write('  <zero_potential type="polynomial" ' +
                      'nderiv="%d" r0="%r" grid="g1">\n' %
                      (self.nderiv0, self.r0))
        else:
            xml.write(('  <zero_potential type="%s" ' +
                       'e0="%r" nderiv="%d" r0="%r" grid="g1">\n') %
                      ('spdfg'[self.l0], self.e0, self.nderiv0, self.r0))

        for x in self.vbar_g:
            print('%r' % x, end=' ', file=xml)
        print('\n  </zero_potential>', file=xml)

        if self.has_corehole:
            print((('  <core_hole_state state="%d%s" ' +
                    'removed="%r" eig="%r" ekin="%r">') %
                   (self.ncorehole, 'spdf'[self.lcorehole],
                    self.fcorehole,
                    self.core_hole_e, self.core_hole_e_kin)), file=xml)
            for x in self.phicorehole_g:
                print('%r' % x, end=' ', file=xml)
            print('\n  </core_hole_state>', file=xml)

        for name, a in [('ae_core_density', self.nc_g),
                        ('pseudo_core_density', self.nct_g),
                        ('ae_core_kinetic_energy_density', self.tauc_g),
                        ('pseudo_core_kinetic_energy_density', self.tauct_g)]:
            print('  <%s grid="g1">\n    ' % name, end=' ', file=xml)
            for x in a:
                print('%r' % x, end=' ', file=xml)
            print('\n  </%s>' % name, file=xml)

        # Print xc-specific data to setup file (used so for KLI and GLLB)
        for name, a in self.extra_xc_data.items():
            newname = 'GLLB_' + name
            print('  <%s grid="g1">\n    ' % newname, end=' ', file=xml)
            for x in a:
                print('%r' % x, end=' ', file=xml)
            print('\n  </%s>' % newname, file=xml)

        for id, l, u, s, q, in zip(self.id_j, l_j, self.phi_jg, self.phit_jg,
                                   self.pt_jg):
            for name, a in [('ae_partial_wave', u),
                            ('pseudo_partial_wave', s),
                            ('projector_function', q)]:
                print('  <%s state="%s" grid="g1">\n    ' % (name, id),
                      end=' ', file=xml)
                for x in a:
                    print('%r' % x, end=' ', file=xml)
                print('\n  </%s>' % name, file=xml)

        print('  <kinetic_energy_differences>', end=' ', file=xml)
        nj = len(self.e_kin_jj)
        for j1 in range(nj):
            print('\n    ', end=' ', file=xml)
            for j2 in range(nj):
                print('%r' % self.e_kin_jj[j1, j2], end=' ', file=xml)
        print('\n  </kinetic_energy_differences>', file=xml)

        if self.X_p is not None:
            print('  <exact_exchange_X_matrix>\n    ', end=' ', file=xml)
            for x in self.X_p:
                print('%r' % x, end=' ', file=xml)
            print('\n  </exact_exchange_X_matrix>', file=xml)

            print('  <exact_exchange core-core="%r"/>' % self.ExxC, file=xml)

        if self.X_pg is not None:
            print('  <yukawa_exchange_X_matrix>\n    ', end=' ', file=xml)
            for x in self.X_pg:
                print('%r' % x, end=' ', file=xml)
            print('\n  </yukawa_exchange_X_matrix>', file=xml)
            print('  <yukawa_exchange gamma="%r"/>' % self.X_gamma, file=xml)
        print('</paw_dataset>', file=xml)

    def build(self, xcfunc, lmax, basis, filter=None):
        from gpaw.setup import Setup
        setup = Setup(self, xcfunc, lmax, basis, filter)
        return setup


def search_for_file(name, world=None):
    """Traverse gpaw setup paths to find file.

    Returns the file path and file contents.  If the file is not
    found, raises RuntimeError."""

    if world is None or world.rank == 0:
        source = None
        filename = None
        for path in setup_paths:
            pattern = os.path.join(path, name)
            filenames = glob(pattern) + glob('%s.gz' % pattern)
            if filenames:
                # The globbing is a hack to grab the 'newest' version if
                # the files are somehow version numbered; then we want the
                # last/newest of the results (used with SG15).  (User must
                # instantiate (UPF)SetupData directly to override.)
                filename = max(filenames)
                assert has_gzip  # Which systems do not have the gzip module?
                if filename.endswith('.gz'):
                    fd = gzip.open(filename)
                else:
                    fd = open(filename, 'rb')
                source = fd.read()
                break

    if world is not None:
        if world.rank == 0:
            broadcast((filename, source), 0, world)
        else:
            filename, source = broadcast(None, 0, world)

    if source is None:
        if name.endswith('basis'):
            _type = 'basis set'
        else:
            _type = 'PAW dataset'
        err = 'Could not find required %s file "%s".' % (_type, name)
        helpful_message = """
You need to set the GPAW_SETUP_PATH environment variable to point to
the directories where PAW dataset and basis files are stored.  See
https://wiki.fysik.dtu.dk/gpaw/install.html#install-paw-datasets
for details."""
        raise RuntimeError('%s\n%s' % (err, helpful_message))

    return filename, source


class PAWXMLParser(xml.sax.handler.ContentHandler):
    def __init__(self, setup):
        xml.sax.handler.ContentHandler.__init__(self)
        self.setup = setup
        self.id = None
        self.data = None

    def parse(self, source=None, world=None):
        setup = self.setup
        if source is None:
            setup.filename, source = search_for_file(setup.stdfilename, world)

        setup.fingerprint = hashlib.md5(source).hexdigest()

        # XXXX There must be a better way!
        # We don't want to look at the dtd now.  Remove it:
        source = re.compile(b'<!DOCTYPE .*?>', re.DOTALL).sub(b'', source, 1)
        xml.sax.parseString(source, self)

        if setup.zero_reference:
            setup.e_total = 0.0
            setup.e_kinetic = 0.0
            setup.e_electrostatic = 0.0
            setup.e_xc = 0.0

    def startElement(self, name, attrs):
        setup = self.setup
        if name == 'paw_setup' or name == 'paw_dataset':
            setup.version = attrs['version']
            assert LooseVersion(setup.version) >= '0.4'
        if name == 'atom':
            Z = float(attrs['Z'])
            setup.Z = int(Z)
            assert setup.Z == Z
            setup.Nc = float(attrs['core'])
            Nv = float(attrs['valence'])
            setup.Nv = int(Nv)
            assert setup.Nv == Nv
        elif name == 'xc_functional':
            if attrs['type'] == 'LDA':
                setup.xcname = 'LDA'
            else:
                setup.xcname = attrs['name']
                if attrs['type'] == 'OFDFT':
                    setup.orbital_free = True
                else:
                    assert attrs['type'] == 'GGA'
        elif name == 'ae_energy':
            setup.e_total = float(attrs['total'])
            setup.e_kinetic = float(attrs['kinetic'])
            setup.e_electrostatic = float(attrs['electrostatic'])
            setup.e_xc = float(attrs['xc'])
        elif name == 'core_energy':
            setup.e_kinetic_core = float(attrs['kinetic'])
        elif name == 'state':
            setup.n_j.append(int(attrs.get('n', -1)))
            setup.l_j.append(int(attrs['l']))
            setup.f_j.append(float(attrs.get('f', 0)))
            setup.eps_j.append(float(attrs['e']))
            setup.rcut_j.append(float(attrs.get('rc', -1)))
            setup.id_j.append(attrs['id'])
            # Compatibility with old setups:
            if LooseVersion(setup.version) < '0.6' and setup.f_j[-1] == 0:
                setup.n_j[-1] = -1
        elif name == 'radial_grid':
            if attrs['eq'] == 'r=a*i/(n-i)':
                beta = float(attrs['a'])
                ng = int(attrs['n'])
                setup.rgd = AERadialGridDescriptor(beta / ng, 1.0 / ng, ng)
            elif attrs['eq'] == 'r=a*i/(1-b*i)':
                a = float(attrs['a'])
                b = float(attrs['b'])
                N = int(attrs['n'])
                setup.rgd = AERadialGridDescriptor(a, b, N)
            elif attrs['eq'] == 'r=a*(exp(d*i)-1)':
                a = float(attrs['a'])
                d = float(attrs['d'])
                istart = int(attrs['istart'])
                iend = int(attrs['iend'])
                assert istart == 0
                setup.rgd = AbinitRadialGridDescriptor(a, d, iend + 1)
            else:
                raise ValueError('Unknown grid:' + attrs['eq'])
        elif name == 'shape_function':
            assert attrs['type'] in {'gauss', 'sinc', 'bessel'}
            setup.shape_function = {'type': attrs['type'],
                                    'rc': float(attrs['rc'])}
        elif name in ['ae_core_density', 'pseudo_core_density',
                      'localized_potential', 'yukawa_exchange_X_matrix',
                      'kinetic_energy_differences', 'exact_exchange_X_matrix',
                      'ae_core_kinetic_energy_density',
                      'pseudo_core_kinetic_energy_density']:
            self.data = []
        elif name.startswith('GLLB_'):
            self.data = []
        elif name in ['ae_partial_wave', 'pseudo_partial_wave']:
            self.data = []
            self.id = attrs['state']
        elif name == 'projector_function':
            self.id = attrs['state']
            self.data = []
        elif name == 'exact_exchange':
            setup.ExxC = float(attrs['core-core'])
        elif name == 'yukawa_exchange':
            setup.X_gamma = float(attrs['gamma'])
        elif name == 'core_hole_state':
            setup.has_corehole = True
            setup.fcorehole = float(attrs['removed'])
            setup.lcorehole = 'spdf'.find(attrs['state'][1])
            setup.core_hole_e = float(attrs['eig'])
            setup.core_hole_e_kin = float(attrs['ekin'])
            self.data = []
        elif name == 'zero_potential':
            if 'type' in attrs:
                setup.r0 = float(attrs['r0'])
                setup.nderiv0 = int(attrs['nderiv'])
                if attrs['type'] == 'polynomial':
                    setup.e0 = None
                    setup.l0 = None
                else:
                    setup.e0 = float(attrs['e0'])
                    setup.l0 = 'spdfg'.find(attrs['type'])
            self.data = []
        elif name == 'generator':
            setup.type = attrs['type']
            setup.generator_version = int(attrs.get('version', '1'))
        else:
            self.data = None

    def characters(self, data):
        if self.data is not None:
            self.data.append(data)

    def endElement(self, name):
        setup = self.setup
        if self.data is None:
            return
        x_g = np.array([float(x) for x in ''.join(self.data).split()])
        if name == 'ae_core_density':
            setup.nc_g = x_g
        elif name == 'pseudo_core_density':
            setup.nct_g = x_g
        elif name == 'kinetic_energy_differences':
            setup.e_kin_jj = x_g
        elif name == 'ae_core_kinetic_energy_density':
            setup.tauc_g = x_g
        elif name == 'pseudo_valence_density':
            setup.nvt_g = x_g
        elif name == 'pseudo_core_kinetic_energy_density':
            if extra_parameters.get('mggapscore') and (x_g == 0).all():
                x = setup.rgd.r_g / 0.7
                x_g = 0.051 * (1 - x**2 * (3 - 2 * x))
                x_g[x > 1] = 0.0
            setup.tauct_g = x_g
        elif name in ['localized_potential', 'zero_potential']:  # XXX
            setup.vbar_g = x_g
        elif name.startswith('GLLB_'):
            # Add setup tags starting with GLLB_ to extra_xc_data. Remove
            # GLLB_ from front of string:
            setup.extra_xc_data[name[5:]] = x_g
        elif name == 'ae_partial_wave':
            j = len(setup.phi_jg)
            assert self.id == setup.id_j[j]
            setup.phi_jg.append(x_g)
        elif name == 'pseudo_partial_wave':
            j = len(setup.phit_jg)
            assert self.id == setup.id_j[j]
            setup.phit_jg.append(x_g)
        elif name == 'projector_function':
            j = len(setup.pt_jg)
            assert self.id == setup.id_j[j]
            setup.pt_jg.append(x_g)
        elif name == 'exact_exchange_X_matrix':
            setup.X_p = x_g
        elif name == 'yukawa_exchange_X_matrix':
            setup.X_pg = x_g
        elif name == 'core_hole_state':
            setup.phicorehole_g = x_g



import sys


class Out:
    def write(self, x):
        sys.__stdout__.write(x)
        raise RuntimeError('not silent')

out, err = sys.stdout, sys.stderr
sys.stdout = sys.stderr = Out()

try:
    from gpaw import GPAW, FermiDirac
    from ase import Atom, Atoms

    a = 5.0
    h = 0.2
    calc = GPAW(h=h, nbands=1, kpts=(1, 1, 1),
                occupations=FermiDirac(width=1e-9),
                xc='PBE',
                txt=None)
    hydrogen = Atoms([Atom('H', (a / 2, a / 2, a / 2), magmom=0)],
                     cell=(a, a, a),
                     calculator=calc)
    f = hydrogen.get_forces()
except Exception:
    sys.stdout = out
    sys.stderr = err
    raise

sys.stdout = out
sys.stderr = err



"""Provide mechanism to broadcast imports from master to other processes.

This reduces file system strain.

Use:

  with broadcast_imports():
      <execute import statements>

This temporarily overrides the Python import mechanism so that

  1) master executes and caches import metadata and code
  2) import metadata and code are broadcast to all processes
  3) other processes execute the import statements from memory

Warning: Do not perform any parallel operations while broadcast imports
are enabled.  Non-master processes assume that they will receive module
data and will crash or deadlock if master sends anything else.
"""


import os
import sys
import marshal
import importlib
import importlib.util
from importlib.machinery import PathFinder, ModuleSpec

import _gpaw


if hasattr(_gpaw, 'Communicator'):
    if '_gpaw' not in sys.builtin_module_names:
        libmpi = os.environ.get('GPAW_MPI', 'libmpi.so')
        import ctypes
        ctypes.CDLL(libmpi, ctypes.RTLD_GLOBAL)

    world = _gpaw.Communicator()
else:
    world = None

paths = {}
sources = {}


def marshal_broadcast(obj):
    if world.rank == 0:
        buf = marshal.dumps(obj)
    else:
        assert obj is None
        buf = None

    buf = _gpaw.globally_broadcast_bytes(buf)
    try:
        return marshal.loads(buf)
    except ValueError as err:
        msg = ('Parallel import failure -- probably received garbage.  '
               'Error was: {}.  This may happen if parallel operations are '
               'performed while parallel imports are enabled.'.format(err))
        raise ImportError(msg)


class BroadcastLoader:
    def __init__(self, spec, module_cache):
        self.module_cache = module_cache
        self.spec = spec

    def load_module(self, fullname):
        if world.rank == 0:
            # Load from file and store in cache:
            code = self.spec.loader.get_code(fullname)
            metadata = (self.spec.submodule_search_locations, self.spec.origin)
            self.module_cache[fullname] = (metadata, code)
            # We could execute the default mechanism to load the module here.
            # Instead we load from cache using our own loader, like on the
            # other cores.

        return self.load_from_cache(fullname)

    def load_from_cache(self, fullname):
        metadata, code = self.module_cache[fullname]
        module = importlib.util.module_from_spec(self.spec)
        origin = metadata[1]
        module.__file__ = origin
        # __package__, __path__, __cached__?
        module.__loader__ = self
        sys.modules[fullname] = module
        exec(code, module.__dict__)
        return module

    def __str__(self):
        return ('<{} for {}:{} [{} modules cached]>'
                .format(self.__class__.__name__,
                        self.spec.name, self.spec.origin,
                        len(self.module_cache)))


class BroadcastImporter:
    def __init__(self):
        self.module_cache = {}

    def find_spec(self, fullname, path=None, target=None):
        if world.rank == 0:
            spec = PathFinder.find_spec(fullname, path, target)
            if spec is None:
                return None

            if spec.loader is None:
                raise ImportError

            code = spec.loader.get_code(fullname)
            if code is None:  # C extensions
                return None

            loader = BroadcastLoader(spec, self.module_cache)
            assert fullname == spec.name

            searchloc = spec.submodule_search_locations
            spec = ModuleSpec(fullname, loader, origin=spec.origin,
                              is_package=searchloc is not None)
            if searchloc is not None:
                spec.submodule_search_locations += searchloc
            return spec
        else:
            if fullname not in self.module_cache:
                # Could this in principle interfere with builtin imports?
                return PathFinder.find_spec(fullname, path, target)

            searchloc, origin = self.module_cache[fullname][0]
            loader = BroadcastLoader(None, self.module_cache)
            spec = ModuleSpec(fullname, loader, origin=origin,
                              is_package=searchloc is not None)
            if searchloc is not None:
                spec.submodule_search_locations += searchloc
            loader.spec = spec  # XXX loader.loader is still None
            return spec

    def broadcast(self):
        if world.rank == 0:
            # print('bcast {} modules'.format(len(self.module_cache)))
            marshal_broadcast(self.module_cache)
        else:
            self.module_cache = marshal_broadcast(None)
            # print('recv {} modules'.format(len(self.module_cache)))

    def enable(self):
        if world is None:
            return

        # There is the question of whether we lose anything by inserting
        # ourselves further on in the meta_path list.  Maybe not, and maybe
        # that is a less violent act.
        sys.meta_path.insert(0, self)
        if world.rank != 0:
            self.broadcast()

    def disable(self):
        if world is None:
            return

        if world.rank == 0:
            self.broadcast()
        self.module_cache = {}
        myself = sys.meta_path.pop(0)
        assert myself is self

    def __enter__(self):
        self.enable()

    def __exit__(self, *args):
        self.disable()


broadcast_imports = BroadcastImporter()


if 0:
    import imp
    import pickle

    def broadcast(obj):
        if world.rank == 0:
            buf = pickle.dumps(obj, pickle.HIGHEST_PROTOCOL)
        else:
            assert obj is None
            buf = None

        buf = _gpaw.globally_broadcast_bytes(buf)
        newobj = pickle.loads(buf)
        return newobj

    class ModuleData:
        def __init__(self, vars, code):
            self.vars = vars
            self.code = code

    class OurFinder:
        def __init__(self, module_cache, module_findcache):
            self.module_cache = module_cache
            self.module_findcache = module_findcache

        def find_module(self, fullname, path):
            if world.rank == 0:
                moduleinfo = imp.find_module(fullname.split('.')[-1], path)
                # print(type(fullname), type(path))
                # print(moduleinfo)
                # if path is not None:
                #     path = tuple(path)
                # self.module_findcache[(fullname, path)] = moduleinfo
            else:
                # if path is not None:
                #     path = tuple(path)
                # moduleinfo = self.module_findcache[(fullname, path)]
                moduleinfo = None
            return OurLoader(moduleinfo, self.module_cache)

    class OurLoader:
        def __init__(self, moduleinfo, module_cache):
            self.moduleinfo = moduleinfo
            self.module_cache = module_cache

        def load_module(self, name):
            if name in sys.modules:
                return sys.modules[name]

            if world.rank == 0:
                return self.load_and_cache(name)
            else:
                return self.load_from_cache(name)

        def load_and_cache(self, name):
            module = self.load_as_normal(name)

            # Some data, like __path__, is not included in the code.
            # We must manually handle these:
            module_vars = {}

            # XXX is this guaranteed to be complete?
            for var in ['__path__', '__package__', '__file__', '__cached__']:
                if hasattr(module, var):
                    module_vars[var] = getattr(module, var)

            # Load module code, if the module actually comes from a file, and
            # the file is a python-file (not e.g. C extensions like .so):
            code = None
            if hasattr(module, '__file__'):
                filename = module.__file__
                if any(filename.endswith(extension)
                       for extension in ['.py', '.pyc', 'pyo']):
                    with open(filename, 'rb') as fd:
                        code = fd.read()

            self.module_cache[name] = ModuleData(module_vars, code)
            return module

        def load_from_cache(self, name):
            module_data = self.module_cache[name]

            if module_data.code is None:
                return self.load_as_normal(name)

            if sys.version_info[0] == 2:
                # Load, ignoring checksum and time stamp.
                # 8 is header length (12 in py3, if we need that someday)
                code = marshal.loads(module_data.code[8:])
            else:
                code = module_data.code

            imp.acquire_lock()  # Required in threaded applications

            print('{} load {}'.format(world.rank, name))
            module = imp.new_module(name)

            # To properly handle circular and submodule imports, we must
            # add the module before executing its code:
            sys.modules[name] = module

            # Set data like __path__, __file__ etc. which are defined
            # by the loader and not the code itself:
            for var in module_data.vars:
                setattr(module, var, module_data.vars[var])

            exec(code, module.__dict__)

            imp.release_lock()
            return module

        def load_as_normal(self, name):
            module = imp.load_module(name, *self.moduleinfo)
            sys.modules[name] = module
            return module

    class BroadCaster:
        def __init__(self):
            self.oldmetapath = None
            self.module_cache = {} if world.rank == 0 else None
            self.module_findcache = {} if world.rank == 0 else None

        def __enter__(self):
            # assert self.oldmetapath is None, self.oldmetapath
            self.oldmetapath = sys.meta_path
            if world.rank != 0:
                # Here we wait for the master process to finish all its
                # imports; the master process will broadcast its results
                # from its __exit__ method, but we slaves need to receive
                # those data in order to start importing.
                self.broadcast()

            # Override standard import finder/loader:
            sys.meta_path = [OurFinder(self.module_cache,
                                       self.module_findcache)]

        def __exit__(self, *args):
            assert len(sys.meta_path) == 1
            sys.meta_path = self.oldmetapath

            # Restore import loader to its former glory:
            if world.rank == 0:
                self.broadcast()

            self.module_cache = {} if world.rank == 0 else None
            self.module_findcache = {} if world.rank == 0 else None

        def broadcast(self):
            if world.rank == 0:
                print('0 send {} modules'.format(len(self.module_cache)))
            else:
                print('rank', world.rank, 'recv', self.module_cache)
            self.module_cache = broadcast(self.module_cache)
            self.module_findcache = broadcast(self.module_findcache)
            if world.rank != 0:
                print('recvd {} modules'.format(len(self.module_cache)))



"""Omega matrix for functionals with Hartree-Fock exchange.

"""
from __future__ import print_function
from math import sqrt

import numpy as np
from numpy.linalg import inv

from ase.units import Hartree
from ase.utils.timing import Timer

import gpaw.mpi as mpi
from gpaw.lrtddft.omega_matrix import OmegaMatrix
from gpaw.pair_density import PairDensity
from gpaw.helmholtz import HelmholtzSolver
from gpaw.utilities.lapack import diagonalize, gemm, sqrt_matrix


class ApmB(OmegaMatrix):

    """Omega matrix for functionals with Hartree-Fock exchange.

    """

    def get_full(self):

        hybrid = ((self.xc is not None) and
                  hasattr(self.xc, 'hybrid') and
                  (self.xc.hybrid > 0.0))
        if self.fullkss.npspins < 2 and hybrid:
            raise RuntimeError('Does not work spin-unpolarized ' +
                               'with hybrids (use nspins=2)')

        if hasattr(self.xc, 'rsf') and (self.xc.rsf == 'Yukawa'):
            self.screened_poissonsolver = HelmholtzSolver(
                k2=-self.xc.omega**2, eps=1e-11, nn=3)
            self.screened_poissonsolver.set_grid_descriptor(self.gd)
        self.paw.timer.start('ApmB RPA')
        self.ApB = self.Om
        self.AmB = self.get_rpa()
        self.paw.timer.stop()

        if self.xc is not None:
            self.paw.timer.start('ApmB XC')
            self.get_xc()  # inherited from OmegaMatrix
            self.paw.timer.stop()

    def get_rpa(self):
        """Calculate RPA and Hartree-fock part of the A+-B matrices."""

        # shorthands
        kss = self.fullkss
        finegrid = self.finegrid
        yukawa = hasattr(self.xc, 'rsf') and (self.xc.rsf == 'Yukawa')

        # calculate omega matrix
        nij = len(kss)
        print('RPAhyb', nij, 'transitions', file=self.txt)

        AmB = np.zeros((nij, nij))
        ApB = self.ApB

        # storage place for Coulomb integrals
        integrals = {}
        if yukawa:
            rsf_integrals = {}
        # setup things for IVOs
        if self.xc.excitation is not None or self.xc.excited != 0:
            sin_tri_weight = 1
            if self.xc.excitation is not None:
                ex_type = self.xc.excitation.lower()
                if ex_type == 'singlet':
                    sin_tri_weight = 2
                elif ex_type == 'triplet':
                    sin_tri_weight = 0
            homo = int(self.paw.get_number_of_electrons() // 2)
            ivo_l = homo - self.xc.excited - 1
        else:
            ivo_l = None

        for ij in range(nij):
            print('RPAhyb kss[' + '%d' % ij + ']=', kss[ij], file=self.txt)

            timer = Timer()
            timer.start('init')
            timer2 = Timer()

            # smooth density including compensation charges
            timer2.start('with_compensation_charges 0')
            rhot_p = kss[ij].with_compensation_charges(
                finegrid != 0)
            timer2.stop()

            # integrate with 1/|r_1-r_2|
            timer2.start('poisson')
            phit_p = np.zeros(rhot_p.shape, rhot_p.dtype)
            self.poisson.solve(phit_p, rhot_p, charge=None)
            timer2.stop()

            timer.stop()
            t0 = timer.get_time('init')
            timer.start(ij)

            if finegrid == 1:
                rhot = kss[ij].with_compensation_charges()
                phit = self.gd.zeros()
                self.restrict(phit_p, phit)
            else:
                phit = phit_p
                rhot = rhot_p

            for kq in range(ij, nij):
                if kq != ij:
                    # smooth density including compensation charges
                    timer2.start('kq with_compensation_charges')
                    rhot = kss[kq].with_compensation_charges(
                        finegrid == 2)
                    timer2.stop()
                pre = self.weight_Kijkq(ij, kq)

                timer2.start('integrate')
                I = self.Coulomb_integral_kss(kss[ij], kss[kq], phit, rhot)
                if kss[ij].spin == kss[kq].spin:
                    name = self.Coulomb_integral_name(kss[ij].i, kss[ij].j,
                                                      kss[kq].i, kss[kq].j,
                                                      kss[ij].spin)
                    integrals[name] = I
                ApB[ij, kq] = pre * I
                timer2.stop()

                if ij == kq:
                    epsij = kss[ij].get_energy() / kss[ij].get_weight()
                    AmB[ij, kq] += epsij
                    ApB[ij, kq] += epsij

            timer.stop()
# timer2.write()
            if ij < (nij - 1):
                print('RPAhyb estimated time left',
                      self.time_left(timer, t0, ij, nij), file=self.txt)

        # add HF parts and apply symmetry
        if hasattr(self.xc, 'hybrid'):
            weight = self.xc.hybrid
        else:
            weight = 0.0
        for ij in range(nij):
            print('HF kss[' + '%d' % ij + ']', file=self.txt)
            timer = Timer()
            timer.start('init')
            timer.stop()
            t0 = timer.get_time('init')
            timer.start(ij)

            i = kss[ij].i
            j = kss[ij].j
            s = kss[ij].spin
            for kq in range(ij, nij):
                if kss[ij].pspin == kss[kq].pspin:
                    k = kss[kq].i
                    q = kss[kq].j
                    ikjq = self.Coulomb_integral_ijkq(i, k, j, q, s, integrals)
                    iqkj = self.Coulomb_integral_ijkq(i, q, k, j, s, integrals)
                    if yukawa:  # Yukawa integrals might be caches
                        ikjq -= self.Coulomb_integral_ijkq(
                            i, k, j, q, s, rsf_integrals, yukawa)
                        iqkj -= self.Coulomb_integral_ijkq(
                            i, q, k, j, s, rsf_integrals, yukawa)
                    ApB[ij, kq] -= weight * (ikjq + iqkj)
                    AmB[ij, kq] -= weight * (ikjq - iqkj)

                ApB[kq, ij] = ApB[ij, kq]
                AmB[kq, ij] = AmB[ij, kq]

            timer.stop()
            if ij < (nij - 1):
                print('HF estimated time left',
                      self.time_left(timer, t0, ij, nij), file=self.txt)

        if ivo_l is not None:
            # IVO RPA after Berman, Kaldor, Chem. Phys. 43 (3) 1979
            # doi: 10.1016/0301-0104(79)85205-2
            l = ivo_l
            for ij in range(nij):
                i = kss[ij].i
                j = kss[ij].j
                s = kss[ij].spin
                for kq in range(ij, nij):
                    if kss[kq].i == i and kss[ij].pspin == kss[kq].pspin:
                        k = kss[kq].i
                        q = kss[kq].j
                        jqll = self.Coulomb_integral_ijkq(j, q, l, l, s,
                                                          integrals)
                        jllq = self.Coulomb_integral_ijkq(j, l, l, q, s,
                                                          integrals)
                        if yukawa:
                            jqll -= self.Coulomb_integral_ijkq(j, q, l, l, s,
                                                               rsf_integrals,
                                                               yukawa)
                            jllq -= self.Coulomb_integral_ijkq(j, l, l, q, s,
                                                               rsf_integrals,
                                                               yukawa)
                        jllq *= sin_tri_weight
                        ApB[ij, kq] += weight * (jqll - jllq)
                        AmB[ij, kq] += weight * (jqll - jllq)
                        ApB[kq, ij] = ApB[ij, kq]
                        AmB[kq, ij] = AmB[ij, kq]
        return AmB

    def Coulomb_integral_name(self, i, j, k, l, spin):
        """return a unique name considering the Coulomb integral
        symmetry"""
        def ij_name(i, j):
            return str(max(i, j)) + ' ' + str(min(i, j))

        # maximal gives the first
        if max(i, j) >= max(k, l):
            base = ij_name(i, j) + ' ' + ij_name(k, l)
        else:
            base = ij_name(k, l) + ' ' + ij_name(i, j)
        return base + ' ' + str(spin)

    def Coulomb_integral_ijkq(self, i, j, k, q, spin, integrals,
                              yukawa=False):
        name = self.Coulomb_integral_name(i, j, k, q, spin)
        if name in integrals:
            return integrals[name]

        # create the Kohn-Sham singles
        kss_ij = PairDensity(self.paw)
        kss_ij.initialize(self.paw.wfs.kpt_u[spin], i, j)
        kss_kq = PairDensity(self.paw)
        kss_kq.initialize(self.paw.wfs.kpt_u[spin], k, q)

        rhot_p = kss_ij.with_compensation_charges(
            self.finegrid != 0)
        phit_p = np.zeros(rhot_p.shape, rhot_p.dtype)
        if yukawa:
            self.screened_poissonsolver.solve(phit_p, rhot_p, charge=None)
        else:
            self.poisson.solve(phit_p, rhot_p, charge=None)

        if self.finegrid == 1:
            phit = self.gd.zeros()
            self.restrict(phit_p, phit)
        else:
            phit = phit_p

        rhot = kss_kq.with_compensation_charges(
            self.finegrid == 2)

        integrals[name] = self.Coulomb_integral_kss(kss_ij, kss_kq,
                                                    phit, rhot,
                                                    yukawa=yukawa)
        return integrals[name]

    def timestring(self, t):
        ti = int(t + .5)
        td = int(ti // 86400)
        st = ''
        if td > 0:
            st += '%d' % td + 'd'
            ti -= td * 86400
        th = int(ti // 3600)
        if th > 0:
            st += '%d' % th + 'h'
            ti -= th * 3600
        tm = int(ti // 60)
        if tm > 0:
            st += '%d' % tm + 'm'
            ti -= tm * 60
        st += '%d' % ti + 's'
        return st

    def map(self, istart=None, jend=None, energy_range=None):
        """Map A+B, A-B matrices according to constraints."""

        map, self.kss = self.get_map(istart, jend, energy_range)
        if map is None:
            ApB = self.ApB.copy()
            AmB = self.AmB.copy()
        else:
            nij = len(self.kss)
            ApB = np.empty((nij, nij))
            AmB = np.empty((nij, nij))
            for ij in range(nij):
                for kq in range(nij):
                    ApB[ij, kq] = self.ApB[map[ij], map[kq]]
                    AmB[ij, kq] = self.AmB[map[ij], map[kq]]

        return ApB, AmB

    def diagonalize(self, istart=None, jend=None, energy_range=None,
                    TDA=False):
        """Evaluate Eigenvectors and Eigenvalues"""

        ApB, AmB = self.map(istart, jend, energy_range)
        nij = len(self.kss)

        if TDA:
            # Tamm-Dancoff approximation (B=0)
            self.eigenvectors = 0.5 * (ApB + AmB)
            eigenvalues = np.zeros((nij))
            diagonalize(self.eigenvectors, eigenvalues)
            self.eigenvalues = eigenvalues ** 2
        else:
            # the occupation matrix
            C = np.empty((nij,))
            for ij in range(nij):
                C[ij] = 1. / self.kss[ij].fij

            S = C * inv(AmB) * C
            S = sqrt_matrix(inv(S).copy())

            # get Omega matrix
            M = np.zeros(ApB.shape)
            gemm(1.0, ApB, S, 0.0, M)
            self.eigenvectors = np.zeros(ApB.shape)
            gemm(1.0, S, M, 0.0, self.eigenvectors)

            self.eigenvalues = np.zeros((nij))
            diagonalize(self.eigenvectors, self.eigenvalues)

    def read(self, filename=None, fh=None):
        """Read myself from a file"""
        if mpi.rank == mpi.MASTER:
            if fh is None:
                f = open(filename, 'r')
            else:
                f = fh

            f.readline()
            nij = int(f.readline())
            ApB = np.zeros((nij, nij))
            for ij in range(nij):
                l = f.readline().split()
                for kq in range(ij, nij):
                    ApB[ij, kq] = float(l[kq - ij])
                    ApB[kq, ij] = ApB[ij, kq]
            self.ApB = ApB

            f.readline()
            nij = int(f.readline())
            AmB = np.zeros((nij, nij))
            for ij in range(nij):
                l = f.readline().split()
                for kq in range(ij, nij):
                    AmB[ij, kq] = float(l[kq - ij])
                    AmB[kq, ij] = AmB[ij, kq]
            self.AmB = AmB

            if fh is None:
                f.close()

    def weight_Kijkq(self, ij, kq):
        """weight for the coupling matrix terms"""
        return 2.

    def write(self, filename=None, fh=None):
        """Write current state to a file."""
        if mpi.rank == mpi.MASTER:
            if fh is None:
                f = open(filename, 'w')
            else:
                f = fh

            f.write('# A+B\n')
            nij = len(self.fullkss)
            f.write('%d\n' % nij)
            for ij in range(nij):
                for kq in range(ij, nij):
                    f.write(' %g' % self.ApB[ij, kq])
                f.write('\n')

            f.write('# A-B\n')
            nij = len(self.fullkss)
            f.write('%d\n' % nij)
            for ij in range(nij):
                for kq in range(ij, nij):
                    f.write(' %g' % self.AmB[ij, kq])
                f.write('\n')

            if fh is None:
                f.close()

    def __str__(self):
        string = '<ApmB> '
        if hasattr(self, 'eigenvalues'):
            string += 'dimension ' + ('%d' % len(self.eigenvalues))
            string += '\neigenvalues: '
            for ev in self.eigenvalues:
                string += ' ' + ('%f' % (sqrt(ev) * Hartree))
        return string



import numpy as np
from ase import Atoms
from gpaw import GPAW
from gpaw.lrtddft import LrTDDFT
from gpaw.inducedfield.inducedfield_lrtddft import LrTDDFTInducedField
from gpaw.poisson import FDPoissonSolver
from gpaw.test import equal

do_print_values = False  # Use this for printing the reference values
poisson_eps = 1e-12
density_eps = 1e-6

# 0) PoissonSolver
poissonsolver = FDPoissonSolver(eps=poisson_eps)

# 1) Ground state calculation with empty states
atoms = Atoms(symbols='Na2',
              positions=[(0, 0, 0), (3.0, 0, 0)],
              pbc=False)
atoms.center(vacuum=3.0)

calc = GPAW(nbands=20, h=0.6, setups={'Na': '1'}, poissonsolver=poissonsolver,
            experimental={'niter_fixdensity': 2},
            convergence={'density': density_eps})
atoms.set_calculator(calc)
energy = atoms.get_potential_energy()
calc.write('na2_gs_casida.gpw', mode='all')

# 2) Casida calculation
calc = GPAW('na2_gs_casida.gpw')
istart = 0
jend = 20
lr = LrTDDFT(calc, xc='LDA', istart=istart, jend=jend)
lr.diagonalize()
lr.write('na2_lr.dat.gz')

# Start from scratch
del lr
del calc
calc = GPAW('na2_gs_casida.gpw')
# calc.initialize_positions()
# calc.set_positions()
lr = LrTDDFT('na2_lr.dat.gz')

# 3) Calculate induced field
frequencies = [1.0, 2.08]  # Frequencies of interest in eV
folding = 'Gauss'          # Folding function
width = 0.1                # Line width for folding in eV
kickdir = 0                # Kick field direction 0, 1, 2 for x, y, z
ind = LrTDDFTInducedField(paw=calc, lr=lr, frequencies=frequencies,
                          folding=folding, width=width, kickdir=kickdir)
ind.calculate_induced_field(gridrefinement=2,
                            from_density='comp',
                            poissonsolver=poissonsolver,
                            extend_N_cd=3 * np.ones((3, 2), int),
                            deextend=True)

# Estimate tolerance (worst case error accumulation)
tol = (len(lr) ** 2 * ind.fieldgd.integrate(ind.fieldgd.zeros() + 1.0) *
       max(density_eps, np.sqrt(poisson_eps)))
# tol = 0.702253185994
if do_print_values:
    print('tol = %.12f' % tol)

# Test
val1 = ind.fieldgd.integrate(ind.Ffe_wg[0])
val2 = ind.fieldgd.integrate(np.abs(ind.Fef_wvg[0][0]))
val3 = ind.fieldgd.integrate(np.abs(ind.Fef_wvg[0][1]))
val4 = ind.fieldgd.integrate(np.abs(ind.Fef_wvg[0][2]))
val5 = ind.fieldgd.integrate(ind.Ffe_wg[1])
val6 = ind.fieldgd.integrate(np.abs(ind.Fef_wvg[1][0]))
val7 = ind.fieldgd.integrate(np.abs(ind.Fef_wvg[1][1]))
val8 = ind.fieldgd.integrate(np.abs(ind.Fef_wvg[1][2]))

equal(val1, 3175.732161495840, tol)
equal(val2, 1700.727018909886, tol)
equal(val3, 1187.130921347186, tol)
equal(val4, 1187.130921347852, tol)
equal(val5, 10957.059193705705, tol)
equal(val6, 6575.777519299762, tol)
equal(val7, 4589.111152993442, tol)
equal(val8, 4589.111152996577, tol)



import numpy as np
from ase import Atoms
from gpaw import GPAW, PW, FermiDirac
from gpaw.mpi import world

ecut = 200
kpoints = [1, 1, 4]
atoms = Atoms('HLi', cell=[6, 6, 3.4], pbc=True,
              positions=[[3, 3, 0], [3, 3, 1.6]])

for xc in ['LDA', 'PBE']:
    def calculate(d, k):
        label = 'gpaw.{xc}.domain{d}.kpt{k}'.format(xc=xc, d=d, k=k)
        atoms.calc = GPAW(mode=PW(ecut),
                          xc=xc,
                          txt=label + '.txt',
                          parallel={'domain': d, 'kpt': k},
                          kpts={'size': kpoints},
                          occupations=FermiDirac(width=0.1))

        def stopcalc():
            atoms.calc.scf.converged = True
        atoms.calc.attach(stopcalc, 4)

        e = atoms.get_potential_energy()
        f = atoms.get_forces()
        s = atoms.get_stress()
        atoms.calc.write(label + '.gpw', mode='all')
        GPAW(label + '.gpw', txt=None)
        return e, f, s

    for d in [1, 2, 4, 8]:
        for k in [1, 2]:
            if d * k > world.size:
                continue
            e2, f2, s2 = calculate(d, k)
            if d + k == 2:
                e1, f1, s1 = e2, f2, s2
            else:
                eerr = abs(e2 - e1)
                ferr = np.abs(f2 - f1).max()
                serr = np.abs(s2 - s1).max()
                if world.rank == 0:
                    print('errs', d, k, eerr, ferr, serr)
                assert eerr < 1e-11, 'bad {} energy: err={}'.format(xc, eerr)
                assert ferr < 3e-11, 'bad {} forces: err={}'.format(xc, ferr)
                assert serr < 1e-11, 'bad {} stress: err={}'.format(xc, serr)



from __future__ import print_function
import numpy as np

from gpaw.grid_descriptor import GridDescriptor
from gpaw.spline import Spline
import gpaw.mpi as mpi
from gpaw.wavefunctions.pw import PWDescriptor, PWLFC


x = 2.0
rc = 3.5
r = np.linspace(0, rc, 100)

n = 40
a = 8.0
cell_cv = np.array([[a, 0.5, -1], [0, a, 2], [-1, 0, a + 1]])
gd = GridDescriptor((n, n, n), cell_cv, comm=mpi.serial_comm)

a_R = gd.empty()
z = np.linspace(0, n, n, endpoint=False)
a_R[:] = 2 + np.sin(2 * np.pi * z / n)

spos_ac = np.array([(0.15, 0.45, 0.95)])

pd = PWDescriptor(45, gd)
a_G = pd.fft(a_R)

s = Spline(0, rc, 2 * x**1.5 / np.pi * np.exp(-x * r**2))
p = Spline(1, rc, 2 * x**1.5 / np.pi * np.exp(-x * r**2))
d = Spline(2, rc, 2 * x**1.5 / np.pi * np.exp(-x * r**2))

lfc = PWLFC([[s, p, d]], pd)
lfc.set_positions(spos_ac)
b_LG = pd.zeros(9)
lfc.add(b_LG, {0: np.eye(9)})
e1 = pd.integrate(a_G, b_LG)
assert abs(lfc.integrate(a_G)[0] - e1).max() < 1e-11

s1 = []
for i in range(9):
    x = [0, 0, 0, 0, 0, 0, 0, 0, 0]
    x[i] = 1
    s1.append(lfc.stress_tensor_contribution(a_G, {0: x}) -
              np.eye(3) * e1[i])

x = 1e-6
for dist in [[[x, 0, 0], [0, 0, 0], [0, 0, 0]],
             [[0, 0, 0], [0, x, 0], [0, 0, 0]],
             [[0, 0, 0], [0, 0, 0], [0, 0, x]],
             [[0, x, 0], [x, 0, 0], [0, 0, 0]],
             [[0, 0, x], [0, 0, 0], [x, 0, 0]],
             [[0, 0, 0], [0, 0, x], [0, x, 0]]]:
    e = dist + np.eye(3)
    c_cv = np.dot(cell_cv, e)
    gd = GridDescriptor((n, n, n), c_cv, comm=mpi.serial_comm)
    pd = PWDescriptor(45, gd)
    aa_G = a_G / np.linalg.det(e)
    lfc = PWLFC([[s, p, d]], pd)
    lfc.set_positions(spos_ac)
    b_LG = pd.zeros(9)
    lfc.add(b_LG, {0: np.eye(9)})
    e2 = pd.integrate(aa_G, b_LG)
    s2 = (e2 - e1) / x
    error = (np.array(s1) * dist).sum(1).sum(1) / x - s2
    print(s2, abs(error).max())
    assert abs(error).max() < 2e-6




import numpy as np

from gpaw.io.tar import Reader


class _FakeFileObject(object):
    def __init__(self, fileparts, partsize):
        self.fileparts = fileparts
        self.partsize = partsize
        self.fileobj = None

        for fileobj in self.fileparts:
            assert fileobj.size == partsize

        self.seek(0)

    def seek(self, pos, whence=0):
        self.part, partpos = divmod(pos, self.partsize)
        self.fileobj = self.fileparts[self.part]
        self.fileobj.seek(partpos, whence)

    def tell(self):
        return self.fileobj.tell() + self.part * self.partsize

    def read(self, size=None):
        self.part, partpos = divmod(self.tell(), self.partsize)

        buf = str()
        n = self.tell()

        # read some of initial part
        partrem = min(self.partsize - partpos, size)
        buf += self.fileobj.read(partrem)
        n += partrem

        # read whole parts
        while size - n > self.partsize:
            self.seek(n)
            buf += self.fileobj.read(self.partsize)
            n += self.partsize

        # read some of final part
        self.seek(n)
        rem = size - n
        buf += self.fileobj.read(rem)

        return buf

    def close(self):
        for fileobj in self.fileparts:
            fileobj.close()


class IncrementalReader(Reader):
    _iterkey = 'niter'
    _partkey = 'part'
    _iterpattern = '/%06d.part'

    def __init__(self, name):
        self.partitions = {}
        Reader.__init__(self, name)
        self.dims[self._partkey] = 1

    def startElement(self, tag, attrs):
        if tag == 'partition':
            name = attrs['name']
            assert name not in self.partitions.keys()
            self.dtypes[name] = attrs['type']
            self.shapes[name] = []
            self.name = name
            self.partitions[name] = tuple()
        else:
            if tag == 'dimension' and self.name in self.partitions.keys():
                if attrs['name'] == self._iterkey:
                    self.partitions[self.name] += (self._partkey,)
                else:
                    self.partitions[self.name] += (attrs['name'],)

            Reader.startElement(self, tag, attrs)

    def get_file_object(self, name, indices):
        if name in self.partitions.keys():
            # The first index is the partition iterable
            if len(indices) == 0:
                return self.get_partition_object(name)

            i, indices = indices[0], indices[1:]
            partshape = [self.dims[dim] for dim in self.partitions[name]]
            name += self._iterpattern % i
            self.shapes[name] = partshape[1:]  # HACK

        return Reader.get_file_object(self, name, indices)

    def get_data_type(self, name):
        if name not in self.dtypes.keys():
            try:
                name, partname = name.rsplit('/', 1)
            except ValueError:
                raise KeyError(name)

            assert name in self.partitions.keys()

        return Reader.get_data_type(self, name)

    def get_partition_object(self, name):
        assert name in self.partitions.keys()

        dtype, type, itemsize = self.get_data_type(name)
        shape = self.shapes[name]
        size = itemsize * np.prod(shape, dtype=int)

        partshape = [self.dims[dim] for dim in self.partitions[name]]
        partsize = itemsize * np.prod(partshape, dtype=int)

        fileobjs = []
        for i in range(self.dims[self._iterkey]):
            fileobj = self.tar.extractfile(name + self._iterpattern % i)
            fileobjs.append(fileobj)

        return _FakeFileObject(fileobjs, partsize), shape, size, dtype



from gpaw.setup import create_setup

for symbol, M, n in [('Pd', 1, 6),  # 4d -> 5s
                     ('Fe', 3, 5),
                     ('V', 3, 5),
                     ('Ti', 2, 5)]:
    s = create_setup(symbol)
    f_si = s.calculate_initial_occupation_numbers(magmom=M,
                                                  hund=False,
                                                  charge=0,
                                                  nspins=2)
    print(f_si)
    magmom = (f_si[0] - f_si[1]).sum()
    assert abs(magmom - M) < 1e-10
    N = ((f_si[0] - f_si[1]) != 0).sum()
    assert n == N, 'Wrong # of values have changed'



from __future__ import print_function
import numpy as np
from ase.build import bulk
from gpaw import GPAW, FermiDirac
from gpaw.response.bse import BSE
from gpaw.test import findpeak, equal

GS = 1
nosym = 1
bse = 1
check = 1

if GS:
    a = 5.431  # From PRB 73,045112 (2006)
    atoms = bulk('Si', 'diamond', a=a)
    atoms.positions -= a / 8
    calc = GPAW(mode='pw',
                kpts={'size': (2, 2, 2), 'gamma': True},
                occupations=FermiDirac(0.001),
                nbands=12,
                convergence={'bands': -4})
    atoms.set_calculator(calc)
    atoms.get_potential_energy()
    calc.write('Si.gpw', 'all')

if bse:
    eshift = 0.8
    bse = BSE('Si.gpw',
              ecut=50.,
              valence_bands=range(4),
              conduction_bands=range(4, 8),
              eshift=eshift,
              nbands=8,
              write_h=False,
              write_v=False)
    w_w, eps_w = bse.get_dielectric_function(filename=None,
                                             eta=0.2,
                                             w_w=np.linspace(0, 10, 2001))
if check:
    w_ = 2.552
    I_ = 421.15
    w, I = findpeak(w_w, eps_w.imag)
    equal(w, w_, 0.01)
    equal(I, I_, 0.1)

if GS and nosym:
    atoms = bulk('Si', 'diamond', a=a)
    calc = GPAW(mode='pw',
                kpts={'size': (2, 2, 2), 'gamma': True},
                occupations=FermiDirac(0.001),
                nbands=12,
                symmetry='off',
                convergence={'bands': -4})
    atoms.set_calculator(calc)
    atoms.get_potential_energy()
    calc.write('Si.gpw', 'all')

if bse and nosym:
    bse = BSE('Si.gpw',
              ecut=50.,
              valence_bands=range(4),
              conduction_bands=range(4, 8),
              eshift=eshift,
              nbands=8,
              write_h=False,
              write_v=False)
    w_w, eps_w = bse.get_dielectric_function(filename=None,
                                             eta=0.2,
                                             w_w=np.linspace(0, 10, 2001))

if check and nosym:
    w, I = findpeak(w_w, eps_w.imag)
    equal(w, w_, 0.01)
    equal(I, I_, 0.1)



from __future__ import print_function
import numpy as np
tol = 1e-8
a = np.eye(3, dtype=complex)
a[1:, 0] = 0.01j
w0 = [0.98585786, 1.0, 1.01414214]
# NumPy's Diagonalize
from numpy.linalg import eigh
w = eigh(a)[0]
print(w)
assert abs(w - w0).max() < tol
# LAPACK's QR Diagonalize
from gpaw.utilities.lapack import diagonalize
diagonalize(a.copy(), w)
print(w)
assert abs(w - w0).max() < tol
# LAPACK's MR3 Diagonalize
# Requires Netlib LAPACK 3.2.1 or later
# from gpaw.utilities.lapack import diagonalize_mr3
# z = np.zeros_like(a)
# diagonalize_mr3(a, w, z)
# print w
# assert abs(w - w0).max() < tol




from ase import Atoms
from gpaw import GPAW
import numpy as np

atoms = Atoms('H2', [(0.0, 0.0, 0.0), 
                     (0.0, 0.0, 3.995)], 
              cell=(4, 4, 4), pbc=True)

calc = GPAW(txt=None)
atoms.set_calculator(calc)
try:
    calc.initialize(atoms)
    calc.set_positions(atoms)
except RuntimeError as err:
    print('got error as expected: {}'.format(err))
else:
    # silly exception where we actually skip the check for older numpies
    if hasattr(np, 'divmod'):
        assert 2 + 2 == 5



"""Check TDDFT ionizations with Yukawa potential."""
from ase.build import molecule
from ase.units import Hartree
from gpaw import GPAW
from gpaw.mpi import world
from gpaw.cluster import Cluster
from gpaw.occupations import FermiDirac
from gpaw.test import equal
from gpaw.eigensolvers import RMMDIIS
from gpaw.lrtddft import LrTDDFT

# h2o = Cluster(molecule('H2O'))
# h2o.set_initial_magnetic_moments([2, -1, -1])
# h2o.minimal_box(3.0, h=0.3)
h2o_plus = Cluster(molecule('H2O'))
h2o_plus.set_initial_magnetic_moments([2, -0.5, -0.5])
h2o_plus.minimal_box(3.0, h=0.35)


def get_paw():
    """Return calculator object."""
    c = {'energy': 0.001, 'eigenstates': 0.001, 'density': 0.001}
    return GPAW(convergence=c, eigensolver=RMMDIIS(),
                xc='LCY-PBE:omega=0.83:unocc=True',
                parallel={'domain': world.size}, h=0.35,
                occupations=FermiDirac(width=0.0, fixmagmom=True))


# calc = get_paw()
# calc.set(txt='H2O_LCY_PBE_083.log')
calc_plus = get_paw()
calc_plus.set(txt='H2O_plus_LCY_PBE_083.log', charge=1)

# h2o.set_calculator(calc)
# e_h2o = h2o.get_potential_energy()
h2o_plus.set_calculator(calc_plus)
e_h2o_plus = h2o_plus.get_potential_energy()
# e_ion = e_h2o_plus - e_h2o
# e_ion = e_h2o_plus - e_h2o
e_ion = 12.5354477071  # from prior calculation

# print(e_ion, 12.62)
# equal(e_ion, 12.62, 0.1)
lr = LrTDDFT(calc_plus, txt='LCY_TDDFT_H2O.log', istart=3, jend=4)
equal(lr.xc.omega, 0.83)
lr.write('LCY_TDDFT_H2O.ex.gz')
# reading is problematic with EXX on more than one core
if world.rank == 0:
    lr2 = LrTDDFT('LCY_TDDFT_H2O.ex.gz')
    lr2.diagonalize()
    equal(lr2.xc.omega, 0.83)

    # for i, ip_i in enumerate([14.74, 18.51]):  # 0.3 grid and all states
    for i, ip_i in enumerate([24.08]):
        ion_i = lr2[i].get_energy() * Hartree + e_ion
        print(ion_i, ip_i)
        equal(ion_i, ip_i, 0.6)



from ase.units import Ha
import os
import gpaw.mpi as mpi
import numpy as np
from gpaw.xc.fxc import KernelWave
from ase.io.aff import affopen

def calculate_kernel(self, nG, ns, iq, cut_G=None):
    self.unit_cells = self.calc.wfs.kd.N_c
    self.tag = self.calc.atoms.get_chemical_formula(mode='hill')

    if self.av_scheme is not None:
        self.tag += '_' + self.av_scheme + '_nspins' + str(self.nspins)

    kd = self.calc.wfs.kd
    self.bzq_qc = kd.get_bz_q_points(first=True)
    U_scc = kd.symmetry.op_scc
    self.ibzq_qc = kd.get_ibz_q_points(self.bzq_qc, U_scc)[0]

    ecut = self.ecut * Ha
    if isinstance(ecut, (float, int)):
        self.ecut_max = ecut
    else:
        self.ecut_max = max(ecut)

    q_empty = None

    if not os.path.isfile('fhxc_%s_%s_%s_%s.ulm'
                          % (self.tag, self.xc,
                             self.ecut_max, iq)):
        q_empty = iq
        
    if self.xc not in ('RPA'):
        if q_empty is not None:
            self.l_l = np.array([1.0])

            if self.linear_kernel:
                kernel = KernelWave(self.calc,
                                    self.xc,
                                    self.ibzq_qc,
                                    self.fd,
                                    None,
                                    q_empty,
                                    None,
                                    self.Eg,
                                    self.ecut_max,
                                    self.tag,
                                    self.timer)

            elif not self.dyn_kernel:
                kernel = KernelWave(self.calc,
                                    self.xc,
                                    self.ibzq_qc,
                                    self.fd,
                                    self.l_l,
                                    q_empty,
                                    None,
                                    self.Eg,
                                    self.ecut_max,
                                    self.tag,
                                    self.timer)

            else:
                kernel = KernelWave(self.calc,
                                    self.xc,
                                    self.ibzq_qc,
                                    self.fd,
                                    self.l_l,
                                    q_empty,
                                    self.omega_w,
                                    self.Eg,
                                    self.ecut_max,
                                    self.tag,
                                    self.timer)

            kernel.calculate_fhxc()
            del kernel

        mpi.world.barrier()

        if self.spin_kernel:
            r = affopen('fhxc_%s_%s_%s_%s.ulm' %
                        (self.tag, self.xc, self.ecut_max, iq))
            fv = r.fhxc_sGsG
        
            if cut_G is not None:
                cut_sG = np.tile(cut_G, ns)
                cut_sG[len(cut_G):] += len(fv) // ns
                fv = fv.take(cut_sG, 0).take(cut_sG, 1)

        else:
            if self.xc == 'RPA':
                fv = np.eye(nG)
            elif self.xc == 'range_RPA':
                raise NotImplementedError
#                    fv = np.exp(-0.25 * (G_G * self.range_rc) ** 2.0)

            elif self.linear_kernel:
                r = affopen('fhxc_%s_%s_%s_%s.ulm' %
                            (self.tag, self.xc, self.ecut_max, iq))
                fv = r.fhxc_sGsG
                
                if cut_G is not None:
                    fv = fv.take(cut_G, 0).take(cut_G, 1)
                        
            elif not self.dyn_kernel:
                # static kernel which does not scale with lambda
                
                r = affopen('fhxc_%s_%s_%s_%s.ulm' %
                            (self.tag, self.xc, self.ecut_max, iq))
                fv = r.fhxc_lGG
            
                if cut_G is not None:
                    fv = fv.take(cut_G, 1).take(cut_G, 2)

            else:  # dynamical kernel
                r = affopen('fhxc_%s_%s_%s_%s.ulm' %
                            (self.tag, self.xc, self.ecut_max, iq))
                fv = r.fhxc_lwGG
                        
                if cut_G is not None:
                    fv = fv.take(cut_G, 2).take(cut_G, 3)
    else:
        fv = np.eye(nG)

    return fv




from gpaw import GPAW, restart, FD
from ase.build import molecule
from gpaw.test import equal
Eini0 = -17.8037610364
energy_eps = 0.0005
esolvers = ['cg', 'rmm-diis', 'dav']

calc = GPAW(xc='LDA',
            h=0.21,
            eigensolver='cg',
            convergence={'eigenstates': 3.5e-5, 'energy': energy_eps},
            mode=FD(force_complex_dtype=True))

mol = molecule('N2')
mol.center(vacuum=3.0)
mol.set_calculator(calc)

Eini = mol.get_potential_energy()
Iini = calc.get_number_of_iterations()
print('%10s: %12.6f eV in %3d iterations' % ('init(cg)', Eini, Iini))
equal(Eini, Eini0, energy_eps * calc.get_number_of_electrons())

calc.write('N2_complex.gpw', mode='all')

for esolver in esolvers:
    mol, calc = restart('N2_complex.gpw')

    assert calc.wfs.dtype == complex
    assert calc.wfs.kpt_u[0].psit_nG.dtype == complex
    
    calc.set(convergence={'eigenstates': 3.5e-9, 'energy': energy_eps})
    calc.set(eigensolver=esolver)
    E = mol.get_potential_energy()
    equal(E, Eini, energy_eps * calc.get_number_of_electrons())



from __future__ import print_function
from ase.build import molecule
from ase.parallel import parprint
from gpaw import GPAW
from gpaw.cluster import Cluster
from gpaw.occupations import FixedOccupations, ZeroKelvin
from gpaw.test import equal

h = 0.4
box = 2
nbands = 2
txt = '-'
txt = None

H2 = Cluster(molecule('H2'))
H2.minimal_box(box, h)
convergence = {'energy':0.01, 'eigenstates':1.e-3, 'density':1.e-2}

if 1:
    # test ZeroKelvin vs FixedOccupations
    c = GPAW(h=h, nbands=nbands, 
             occupations=ZeroKelvin(True), 
             convergence=convergence,
             txt=txt)
    H2.set_calculator(c)
    E_zk = H2.get_potential_energy()

    c = GPAW(h=h, nbands=nbands, 
             occupations=FixedOccupations([[2, 0]]), 
             convergence=convergence,
             txt=txt)
    H2.set_calculator(c)
    E_fo = H2.get_potential_energy()
    parprint(E_zk, E_fo)
    equal(E_zk, E_fo, 1.e-10)

if 1:
    # test spin-paired vs spin-polarized
    c = GPAW(h=h, nbands=nbands, 
             occupations=FixedOccupations([[1,1]]), 
             convergence=convergence,
             txt=txt)
    H2.set_calculator(c)
    E_ns = H2.get_potential_energy()
if 1:
    c = GPAW(h=h, nbands=nbands, spinpol=True,
             occupations=FixedOccupations([[0.5, 0.5]] * 2), 
             convergence=convergence,
             txt=txt)
    H2.set_calculator(c)
    E_sp = H2.get_potential_energy()
    parprint(E_ns, E_sp)
    equal(E_ns, E_sp, 1.e-6)



# Copyright (C) 2003  CAMP
# Please see the accompanying LICENSE file for further information.
from __future__ import print_function
from gpaw.grid_descriptor import GridDescriptor
from gpaw.transformers import Transformer
import time


n = 6
gda = GridDescriptor((n,n,n))
gdb = gda.refine()
gdc = gdb.refine()
a = gda.zeros()
b = gdb.zeros()
c = gdc.zeros()

inter = Transformer(gdb, gdc, 2).apply
restr = Transformer(gdb, gda, 2).apply

t = time.clock()
for i in range(8*300):
    inter(b, c)
print(time.clock() - t)

t = time.clock()
for i in range(8*3000):
    restr(b, a)
print(time.clock() - t)



import numpy as np

from gpaw.poisson import FDPoissonSolver as PoissonSolver
from gpaw.poisson_extended import ExtendedPoissonSolver
from gpaw.grid_descriptor import GridDescriptor

from gpaw.test import equal

# def equal(x, y, tol=0):
#     print '%.10e vs %.10e at %.10e' % (x, y, tol)


# Model grid
N_c = (16, 16, 3 * 16)
cell_cv = (1, 1, 3)
gd = GridDescriptor(N_c, cell_cv, False)

# Construct model density
coord_vg = gd.get_grid_point_coordinates()
x_g = coord_vg[0, :]
y_g = coord_vg[1, :]
z_g = coord_vg[2, :]
rho_g = gd.zeros()
for z0 in [1, 2]:
    rho_g += 10 * (z_g - z0) * \
        np.exp(-20 * np.sum((coord_vg.T - np.array([.5, .5, z0])).T**2,
                            axis=0))

poissoneps = 1e-20

do_plot = False

if do_plot:
    big_rho_g = gd.collect(rho_g)
    if gd.comm.rank == 0:
        import matplotlib.pyplot as plt
        fig, ax_ij = plt.subplots(3, 4, figsize=(20, 10))
        ax_i = ax_ij.ravel()
        ploti = 0
        Ng_c = gd.get_size_of_global_array()
        plt.sca(ax_i[ploti])
        ploti += 1
        plt.pcolormesh(big_rho_g[Ng_c[0] / 2])
        plt.sca(ax_i[ploti])
        ploti += 1
        plt.plot(big_rho_g[Ng_c[0] / 2, Ng_c[1] / 2])


def plot_phi(phi_g):
    if do_plot:
        big_phi_g = gd.collect(phi_g)
        if gd.comm.rank == 0:
            global ploti
            plt.sca(ax_i[ploti])
            ploti += 1
            plt.pcolormesh(big_phi_g[Ng_c[0] / 2])
            plt.sca(ax_i[ploti])
            ploti += 1
            plt.plot(big_phi_g[Ng_c[0] / 2, Ng_c[1] / 2])
            plt.ylim(np.array([-1, 1]) * 0.15)


def poisson_solve(gd, rho_g, poisson):
    phi_g = gd.zeros()
    npoisson = poisson.solve(phi_g, rho_g)
    return phi_g, npoisson


def compare(phi1_g, phi2_g, val):
    big_phi1_g = gd.collect(phi1_g)
    big_phi2_g = gd.collect(phi2_g)
    if gd.comm.rank == 0:
        equal(np.max(np.absolute(big_phi1_g - big_phi2_g)),
              val, np.sqrt(poissoneps))


# Get reference from default poissonsolver
poisson = PoissonSolver(eps=poissoneps)
poisson.set_grid_descriptor(gd)
phiref_g, npoisson = poisson_solve(gd, rho_g, poisson)

# Test agreement with default
poisson = ExtendedPoissonSolver(eps=poissoneps)
poisson.set_grid_descriptor(gd)
phi_g, npoisson = poisson_solve(gd, rho_g, poisson)
plot_phi(phi_g)
compare(phi_g, phiref_g, 0.0)

# Test moment_corrections=int
poisson = ExtendedPoissonSolver(eps=poissoneps, moment_corrections=4)
poisson.set_grid_descriptor(gd)
phi_g, npoisson = poisson_solve(gd, rho_g, poisson)
plot_phi(phi_g)
compare(phi_g, phiref_g, 4.1182101206e-02)

# Test moment_corrections=list
poisson = ExtendedPoissonSolver(eps=poissoneps,
    moment_corrections=[{'moms': range(4), 'center': [.5, .5, 1]},
                        {'moms': range(4), 'center': [.5, .5, 2]}])
poisson.set_grid_descriptor(gd)
phi_g, npoisson = poisson_solve(gd, rho_g, poisson)
plot_phi(phi_g)
compare(phi_g, phiref_g, 2.7569628594e-02)

# Test extendedgpts
poisson = ExtendedPoissonSolver(eps=poissoneps,
                                extended={'gpts': (24, 24, 3 * 24),
                                          'useprev': False})
poisson.set_grid_descriptor(gd)
phi_g, npoisson = poisson_solve(gd, rho_g, poisson)
plot_phi(phi_g)
compare(phi_g, phiref_g, 2.5351851105e-02)

# Test extendedgpts + moment_corrections, extendedhistory=False
poisson = ExtendedPoissonSolver(eps=poissoneps,
    extended={'gpts': (24, 24, 3 * 24),
              'useprev': False},
    moment_corrections=[{'moms': range(4), 'center': [.5, .5, 1]},
                        {'moms': range(4), 'center': [.5, .5, 2]}])
poisson.set_grid_descriptor(gd)
phi_g, npoisson = poisson_solve(gd, rho_g, poisson)
plot_phi(phi_g)
phi2_g, npoisson2 = poisson_solve(gd, rho_g, poisson)
equal(npoisson, npoisson2)
compare(phi_g, phi2_g, 0.0)
compare(phi_g, phiref_g, 2.75205170866e-02)

# Test extendedgpts + moment_corrections,
# extendedhistory=True
poisson = ExtendedPoissonSolver(eps=poissoneps,
    extended={'gpts': (24, 24, 3 * 24),
              'useprev': True},
    moment_corrections=[{'moms': range(4), 'center': [.5, .5, 1]},
                        {'moms': range(4), 'center': [.5, .5, 2]}])
poisson.set_grid_descriptor(gd)
phi_g, npoisson = poisson_solve(gd, rho_g, poisson)
phi2_g, npoisson2 = poisson_solve(gd, rho_g, poisson)
# The second run should use the old value -> niter=1
equal(npoisson2, 1)
# There is a slight difference in values
# because one more iteration in phi2_g
compare(phi_g, phi2_g, 0.0)


if do_plot:
    if gd.comm.rank == 0:
        plt.show()



# -*- coding: utf-8 -*-
# Copyright (C) 2003-2015  CAMP
# Please see the accompanying LICENSE file for further information.

"""This module defines a Hamiltonian."""
from __future__ import division

import functools

import numpy as np
from ase.units import Ha

from gpaw.arraydict import ArrayDict
from gpaw.external import create_external_potential
from gpaw.hubbard import hubbard
from gpaw.lfc import LFC
from gpaw.poisson import PoissonSolver
from gpaw.spinorbit import soc
from gpaw.transformers import Transformer
from gpaw.utilities import (unpack, pack2, unpack_atomic_matrices,
                            pack_atomic_matrices)
from gpaw.utilities.debug import frozen
from gpaw.utilities.partition import AtomPartition


ENERGY_NAMES = ['e_kinetic', 'e_coulomb', 'e_zero', 'e_external', 'e_xc',
                'e_entropy', 'e_total_free', 'e_total_extrapolated']


def apply_non_local_hamilton(dH_asp, collinear, P, out=None):
    if out is None:
        out = P.new()
    for a, I1, I2 in P.indices:
        if collinear:
            dH_ii = unpack(dH_asp[a][P.spin])
            out.array[:, I1:I2] = np.dot(P.array[:, I1:I2], dH_ii)
        else:
            dH_xp = dH_asp[a]
            # We need the transpose because
            # we are dotting from the left
            dH_ii = unpack(dH_xp[0]).T
            dH_vii = [unpack(dH_p).T for dH_p in dH_xp[1:]]
            out.array[:, 0, I1:I2] = (np.dot(P.array[:, 0, I1:I2],
                                             dH_ii + dH_vii[2]) +
                                      np.dot(P.array[:, 1, I1:I2],
                                             dH_vii[0] - 1j * dH_vii[1]))
            out.array[:, 1, I1:I2] = (np.dot(P.array[:, 1, I1:I2],
                                             dH_ii - dH_vii[2]) +
                                      np.dot(P.array[:, 0, I1:I2],
                                             dH_vii[0] + 1j * dH_vii[1]))
    return out


@frozen
class Hamiltonian:

    def __init__(self, gd, finegd, nspins, collinear, setups, timer, xc, world,
                 redistributor, vext=None):
        self.gd = gd
        self.finegd = finegd
        self.nspins = nspins
        self.collinear = collinear
        self.setups = setups
        self.timer = timer
        self.xc = xc
        self.world = world
        self.redistributor = redistributor

        self.ncomponents = self.nspins if self.collinear else 1 + 3

        self.atomdist = None
        self.dH_asp = None
        self.vt_xG = None
        self.vt_sG = None
        self.vt_vG = None
        self.vHt_g = None
        self.vt_xg = None
        self.vt_sg = None
        self.vt_vg = None
        self.atom_partition = None

        # Energy contributioons that sum up to e_total_free:
        self.e_kinetic = None
        self.e_coulomb = None
        self.e_zero = None
        self.e_external = None
        self.e_xc = None
        self.e_entropy = None

        self.e_total_free = None
        self.e_total_extrapolated = None
        self.e_kinetic0 = None

        self.ref_vt_sG = None
        self.ref_dH_asp = None

        if isinstance(vext, dict):
            vext = create_external_potential(**vext)
        self.vext = vext  # external potential

        self.positions_set = False
        self.spos_ac = None
        self.soc = False

    @property
    def dH(self):
        return functools.partial(apply_non_local_hamilton,
                                 self.dH_asp, self.collinear)

    def update_atomic_hamiltonians(self, value):
        if isinstance(value, dict):
            dtype = complex if self.soc else float
            tmp = self.setups.empty_atomic_matrix(self.ncomponents,
                                                  self.atom_partition, dtype)
            tmp.update(value)
            value = tmp
        assert isinstance(value, ArrayDict) or value is None, type(value)
        if value is not None:
            value.check_consistency()
        self.dH_asp = value

    def __str__(self):
        s = 'Hamiltonian:\n'
        s += ('  XC and Coulomb potentials evaluated on a {0}*{1}*{2} grid\n'
              .format(*self.finegd.N_c))
        s += '  Using the %s Exchange-Correlation functional\n' % self.xc.name
        # We would get the description of the XC functional here,
        # except the thing has probably not been fully initialized yet.
        if self.vext is not None:
            s += '  External potential:\n    {0}\n'.format(self.vext)
        return s

    def summary(self, fermilevel, log):
        log('Energy contributions relative to reference atoms:',
            '(reference = {0:.6f})\n'.format(self.setups.Eref * Ha))

        energies = [('Kinetic:      ', self.e_kinetic),
                    ('Potential:    ', self.e_coulomb),
                    ('External:     ', self.e_external),
                    ('XC:           ', self.e_xc),
                    ('Entropy (-ST):', self.e_entropy),
                    ('Local:        ', self.e_zero)]

        for name, e in energies:
            log('%-14s %+11.6f' % (name, Ha * e))

        log('--------------------------')
        log('Free energy:   %+11.6f' % (Ha * self.e_total_free))
        log('Extrapolated:  %+11.6f' % (Ha * self.e_total_extrapolated))
        log()
        self.xc.summary(log)

        try:
            correction = self.poisson.correction
        except AttributeError:
            pass
        else:
            c = self.poisson.c  # index of axis perpendicular to dipole-layer
            if not self.gd.pbc_c[c]:
                # zero boundary conditions
                vacuum = 0.0
            else:
                v_q = self.pd3.gather(self.vHt_q)
                if self.pd3.comm.rank == 0:
                    axes = (c, (c + 1) % 3, (c + 2) % 3)
                    v_g = self.pd3.ifft(v_q, local=True).transpose(axes)
                    vacuum = v_g[0].mean()
                else:
                    vacuum = np.nan

            wf1 = (vacuum - fermilevel + correction) * Ha
            wf2 = (vacuum - fermilevel - correction) * Ha
            log('Dipole-layer corrected work functions: {:.6f}, {:.6f} eV'
                .format(wf1, wf2))
            log()

    def set_positions_without_ruining_everything(self, spos_ac,
                                                 atom_partition):
        self.spos_ac = spos_ac
        rank_a = atom_partition.rank_a

        # If both old and new atomic ranks are present, start a blank dict if
        # it previously didn't exist but it will needed for the new atoms.
        # XXX what purpose does this serve?  In what case does it happen?
        # How would one even go about figuring it out?  Why does it all have
        # to be so unreadable? -Ask
        #
        if (self.atom_partition is not None and
            self.dH_asp is None and (rank_a == self.gd.comm.rank).any()):
            self.update_atomic_hamiltonians({})

        if self.atom_partition is not None and self.dH_asp is not None:
            self.timer.start('Redistribute')
            self.dH_asp.redistribute(atom_partition)
            self.timer.stop('Redistribute')

        self.atom_partition = atom_partition
        self.atomdist = self.redistributor.get_atom_distributions(spos_ac)

    def set_positions(self, spos_ac, atom_partition):
        self.vbar.set_positions(spos_ac, atom_partition)
        self.xc.set_positions(spos_ac)
        self.set_positions_without_ruining_everything(spos_ac, atom_partition)
        self.positions_set = True

    def initialize(self):
        self.vt_xg = self.finegd.empty(self.ncomponents)
        self.vt_sg = self.vt_xg[:self.nspins]
        self.vt_vg = self.vt_xg[self.nspins:]
        self.vHt_g = self.finegd.zeros()
        self.vt_xG = self.gd.empty(self.ncomponents)
        self.vt_sG = self.vt_xG[:self.nspins]
        self.vt_vG = self.vt_xG[self.nspins:]

    def update(self, density):
        """Calculate effective potential.

        The XC-potential and the Hartree potential are evaluated on
        the fine grid, and the sum is then restricted to the coarse
        grid."""

        self.timer.start('Hamiltonian')
        if self.vt_sg is None:
            with self.timer('Initialize Hamiltonian'):
                self.initialize()

        finegrid_energies = self.update_pseudo_potential(density)
        coarsegrid_e_kinetic = self.calculate_kinetic_energy(density)

        with self.timer('Calculate atomic Hamiltonians'):
            W_aL = self.calculate_atomic_hamiltonians(density)

        atomic_energies = self.update_corrections(density, W_aL)

        # Make energy contributions summable over world:
        finegrid_energies *= self.finegd.comm.size / self.world.size
        coarsegrid_e_kinetic *= self.gd.comm.size / self.world.size
        # (careful with array orderings/contents)
        energies = atomic_energies  # kinetic, coulomb, zero, external, xc
        energies[1:] += finegrid_energies  # coulomb, zero, external, xc
        energies[0] += coarsegrid_e_kinetic  # kinetic

        with self.timer('Communicate'):  # time possible load imbalance
            self.world.sum(energies)

        (self.e_kinetic0, self.e_coulomb, self.e_zero,
         self.e_external, self.e_xc) = energies

        self.timer.stop('Hamiltonian')

    def update_corrections(self, dens, W_aL):
        self.timer.start('Atomic')
        self.update_atomic_hamiltonians(None)  # XXXX

        e_kinetic = 0.0
        e_coulomb = 0.0
        e_zero = 0.0
        e_external = 0.0
        e_xc = 0.0

        D_asp = self.atomdist.to_work(dens.D_asp)
        dtype = complex if self.soc else float
        dH_asp = self.setups.empty_atomic_matrix(self.ncomponents,
                                                 D_asp.partition, dtype)

        for a, D_sp in D_asp.items():
            W_L = W_aL[a]
            setup = self.setups[a]

            if self.nspins == 2:
                D_p = D_sp.sum(0)
            else:
                D_p = D_sp[0]
            dH_p = (setup.K_p + setup.M_p +
                    setup.MB_p + 2.0 * np.dot(setup.M_pp, D_p) +
                    np.dot(setup.Delta_pL, W_L))
            e_kinetic += np.dot(setup.K_p, D_p) + setup.Kc
            e_zero += setup.MB + np.dot(setup.MB_p, D_p)
            e_coulomb += setup.M + np.dot(D_p, (setup.M_p +
                                                np.dot(setup.M_pp, D_p)))

            if self.soc:
                dH_vii = soc(setup, self.xc, D_sp)
                dH_sp = np.zeros_like(D_sp, dtype=complex)
                dH_sp[1:] = pack2(dH_vii)
            else:
                dH_sp = np.zeros_like(D_sp)

            if setup.HubU is not None:
                #assert self.collinear
                eU, dHU_sp = hubbard(setup, D_sp)
                e_xc += eU
                dH_sp += dHU_sp

            dH_sp[:self.nspins] += dH_p

            if self.vext and self.vext.get_name() == 'CDFTPotential':
                # cDFT atomic hamiltonian, eq. 25
                # energy correction added in cDFT main
                h_cdft_a, h_cdft_b = self.vext.get_atomic_hamiltonians(
                    setups=setup.Delta_pL[:,0], atom = a)

                dH_sp[0] += h_cdft_a
                dH_sp[1] += h_cdft_b

            if self.ref_dH_asp:
                assert self.collinear
                dH_sp += self.ref_dH_asp[a]

            dH_asp[a] = dH_sp

        self.timer.start('XC Correction')
        for a, D_sp in D_asp.items():
            e_xc += self.xc.calculate_paw_correction(self.setups[a], D_sp,
                    dH_asp[a], a=a)
        self.timer.stop('XC Correction')

        for a, D_sp in D_asp.items():
            e_kinetic -= (D_sp * dH_asp[a]).sum().real

        self.update_atomic_hamiltonians(self.atomdist.from_work(dH_asp))
        self.timer.stop('Atomic')

        # Make corrections due to non-local xc:
        # self.Enlxc = 0.0  # XXXxcfunc.get_non_local_energy()
        e_kinetic += self.xc.get_kinetic_energy_correction() / self.world.size

        return np.array([e_kinetic, e_coulomb, e_zero, e_external, e_xc])

    def get_energy(self, occ):
        self.e_kinetic = self.e_kinetic0 + occ.e_band
        self.e_entropy = occ.e_entropy

        self.e_total_free = (self.e_kinetic + self.e_coulomb +
                             self.e_external + self.e_zero + self.e_xc +
                             self.e_entropy)
        self.e_total_extrapolated = occ.extrapolate_energy_to_zero_width(
            self.e_total_free)

        if 0:
            print(self.e_total_free,
                  self.e_total_extrapolated,
                  self.e_kinetic,
                  self.e_kinetic0,
                  self.e_coulomb,
                  self.e_external,
                  self.e_zero,
                  self.e_xc,
                  self.e_entropy)
            1 / 0

        return self.e_total_free

    def linearize_to_xc(self, new_xc, density):
        # Store old hamiltonian
        ref_vt_sG = self.vt_sG.copy()
        ref_dH_asp = self.dH_asp.copy()
        self.xc = new_xc
        self.xc.set_positions(self.spos_ac)
        self.update(density)

        ref_vt_sG -= self.vt_sG
        for a, dH_sp in self.dH_asp.items():
            ref_dH_asp[a] -= dH_sp
        self.ref_vt_sG = ref_vt_sG
        self.ref_dH_asp = self.atomdist.to_work(ref_dH_asp)

    def calculate_forces(self, dens, F_av):
        ghat_aLv = dens.ghat.dict(derivative=True)
        nct_av = dens.nct.dict(derivative=True)
        vbar_av = self.vbar.dict(derivative=True)

        self.calculate_forces2(dens, ghat_aLv, nct_av, vbar_av)
        F_coarsegrid_av = np.zeros_like(F_av)

        # Force from compensation charges:
        _Q, Q_aL = dens.calculate_multipole_moments()
        for a, dF_Lv in ghat_aLv.items():
            F_av[a] += np.dot(Q_aL[a], dF_Lv)

        # Force from smooth core charge:
        for a, dF_v in nct_av.items():
            F_coarsegrid_av[a] += dF_v[0]

        # Force from zero potential:
        for a, dF_v in vbar_av.items():
            F_av[a] += dF_v[0]

        self.xc.add_forces(F_av)
        self.gd.comm.sum(F_coarsegrid_av, 0)
        self.finegd.comm.sum(F_av, 0)
        if self.vext:
            if self.vext.get_name()=='CDFTPotential':
                F_av += self.vext.get_cdft_forces()
        F_av += F_coarsegrid_av

    def apply_local_potential(self, psit_nG, Htpsit_nG, s):
        """Apply the Hamiltonian operator to a set of vectors.

        XXX Parameter description is deprecated!

        Parameters:

        a_nG: ndarray
            Set of vectors to which the overlap operator is applied.
        b_nG: ndarray, output
            Resulting H times a_nG vectors.
        kpt: KPoint object
            k-point object defined in kpoint.py.
        calculate_projections: bool
            When True, the integrals of projector times vectors
            P_ni = <p_i | a_nG> are calculated.
            When False, existing P_uni are used
        local_part_only: bool
            When True, the non-local atomic parts of the Hamiltonian
            are not applied and calculate_projections is ignored.

        """
        vt_G = self.vt_sG[s]
        if psit_nG.ndim == 3:
            Htpsit_nG += psit_nG * vt_G
        else:
            for psit_G, Htpsit_G in zip(psit_nG, Htpsit_nG):
                Htpsit_G += psit_G * vt_G

    def apply(self, a_xG, b_xG, wfs, kpt, calculate_P_ani=True):
        """Apply the Hamiltonian operator to a set of vectors.

        Parameters:

        a_nG: ndarray
            Set of vectors to which the overlap operator is applied.
        b_nG: ndarray, output
            Resulting S times a_nG vectors.
        wfs: WaveFunctions
            Wave-function object defined in wavefunctions.py
        kpt: KPoint object
            k-point object defined in kpoint.py.
        calculate_P_ani: bool
            When True, the integrals of projector times vectors
            P_ni = <p_i | a_nG> are calculated.
            When False, existing P_ani are used

        """

        wfs.kin.apply(a_xG, b_xG, kpt.phase_cd)
        self.apply_local_potential(a_xG, b_xG, kpt.s)
        shape = a_xG.shape[:-3]
        P_axi = wfs.pt.dict(shape)

        if calculate_P_ani:  # TODO calculate_P_ani=False is experimental
            wfs.pt.integrate(a_xG, P_axi, kpt.q)
        else:
            for a, P_ni in kpt.P_ani.items():
                P_axi[a][:] = P_ni

        for a, P_xi in P_axi.items():
            dH_ii = unpack(self.dH_asp[a][kpt.s])
            P_axi[a] = np.dot(P_xi, dH_ii)
        wfs.pt.add(b_xG, P_axi, kpt.q)

    def get_xc_difference(self, xc, density):
        """Calculate non-selfconsistent XC-energy difference."""
        if density.nt_sg is None:
            density.interpolate_pseudo_density()
        nt_sg = density.nt_sg
        if hasattr(xc, 'hybrid'):
            xc.calculate_exx()
        finegd_e_xc = xc.calculate(density.finegd, nt_sg)
        D_asp = self.atomdist.to_work(density.D_asp)
        atomic_e_xc = 0.0
        for a, D_sp in D_asp.items():
            setup = self.setups[a]
            atomic_e_xc += xc.calculate_paw_correction(setup, D_sp, a=a)
        e_xc = finegd_e_xc + self.world.sum(atomic_e_xc)
        return e_xc - self.e_xc

    def estimate_memory(self, mem):
        nbytes = self.gd.bytecount()
        nfinebytes = self.finegd.bytecount()
        arrays = mem.subnode('Arrays', 0)
        arrays.subnode('vHt_g', nfinebytes)
        arrays.subnode('vt_sG', self.nspins * nbytes)
        arrays.subnode('vt_sg', self.nspins * nfinebytes)
        self.xc.estimate_memory(mem.subnode('XC'))
        self.poisson.estimate_memory(mem.subnode('Poisson'))
        self.vbar.estimate_memory(mem.subnode('vbar'))

    def write(self, writer):
        # Write all eneriges:
        for name in ENERGY_NAMES:
            energy = getattr(self, name)
            if energy is not None:
                energy *= Ha
            writer.write(name, energy)

        writer.write(
            potential=self.gd.collect(self.vt_xG) * Ha,
            atomic_hamiltonian_matrices=pack_atomic_matrices(self.dH_asp) * Ha)

        self.xc.write(writer.child('xc'))

        if hasattr(self.poisson, 'write'):
            self.poisson.write(writer.child('poisson'))

    def read(self, reader):
        h = reader.hamiltonian

        # Read all energies:
        for name in ENERGY_NAMES:
            energy = h.get(name)
            if energy is not None:
                energy /= reader.ha
            setattr(self, name, energy)

        # Read pseudo potential on the coarse grid
        # and broadcast on kpt/band comm:
        self.initialize()
        self.gd.distribute(h.potential / reader.ha, self.vt_xG)

        self.atom_partition = AtomPartition(self.gd.comm,
                                            np.zeros(len(self.setups), int),
                                            name='hamiltonian-init-serial')

        # Read non-local part of hamiltonian
        self.update_atomic_hamiltonians({})
        dH_sP = h.atomic_hamiltonian_matrices / reader.ha

        if self.gd.comm.rank == 0:
            self.update_atomic_hamiltonians(
                unpack_atomic_matrices(dH_sP, self.setups))

        if hasattr(self.poisson, 'read'):
            self.poisson.read(reader)
            self.poisson.set_grid_descriptor(self.finegd)


class RealSpaceHamiltonian(Hamiltonian):
    def __init__(self, gd, finegd, nspins, collinear, setups, timer, xc, world,
                 vext=None,
                 psolver=None, stencil=3, redistributor=None):
        Hamiltonian.__init__(self, gd, finegd, nspins, collinear,
                             setups, timer, xc,
                             world, vext=vext,
                             redistributor=redistributor)

        # Solver for the Poisson equation:
        if psolver is None:
            psolver = {}
        if isinstance(psolver, dict):
            psolver = PoissonSolver(**psolver)
        self.poisson = psolver
        self.poisson.set_grid_descriptor(self.finegd)

        # Restrictor function for the potential:
        self.restrictor = Transformer(self.finegd, self.redistributor.aux_gd,
                                      stencil)
        self.restrict = self.restrictor.apply

        self.vbar = LFC(self.finegd, [[setup.vbar] for setup in setups],
                        forces=True)

        self.vbar_g = None
        self.npoisson = None

    def restrict_and_collect(self, a_xg, b_xg=None, phases=None):
        if self.redistributor.enabled:
            tmp_xg = self.restrictor.apply(a_xg, output=None, phases=phases)
            b_xg = self.redistributor.collect(tmp_xg, b_xg)
        else:
            b_xg = self.restrictor.apply(a_xg, output=b_xg, phases=phases)
        return b_xg

    def __str__(self):
        s = Hamiltonian.__str__(self)

        degree = self.restrictor.nn * 2 - 1
        name = ['linear', 'cubic', 'quintic', 'heptic'][degree // 2]
        s += ('  Interpolation: tri-%s ' % name +
              '(%d. degree polynomial)\n' % degree)
        s += '  Poisson solver: %s' % self.poisson.get_description()
        return s

    def set_positions(self, spos_ac, rank_a):
        Hamiltonian.set_positions(self, spos_ac, rank_a)
        if self.vbar_g is None:
            self.vbar_g = self.finegd.empty()
        self.vbar_g[:] = 0.0
        self.vbar.add(self.vbar_g)

    def update_pseudo_potential(self, dens):
        self.timer.start('vbar')
        e_zero = self.finegd.integrate(self.vbar_g, dens.nt_g,
                                       global_integral=False)

        vt_g = self.vt_sg[0]
        vt_g[:] = self.vbar_g
        self.timer.stop('vbar')

        e_external = 0.0
        if self.vext is not None:
            if self.vext.get_name() == 'CDFTPotential':
                vext_g = self.vext.get_potential(self.finegd).copy()
                e_external += self.vext.get_cdft_external_energy(dens,
                        self.nspins, vext_g, vt_g, self.vbar_g, self.vt_sg)

            else:
                vext_g = self.vext.get_potential(self.finegd)
                vt_g += vext_g
                e_external = self.finegd.integrate(vext_g, dens.rhot_g,
                                               global_integral=False)

        if self.nspins == 2:
            self.vt_sg[1] = vt_g

        self.timer.start('XC 3D grid')
        e_xc = self.xc.calculate(self.finegd, dens.nt_sg, self.vt_sg)
        e_xc /= self.finegd.comm.size
        self.timer.stop('XC 3D grid')

        self.timer.start('Poisson')
        # npoisson is the number of iterations:
        self.npoisson = self.poisson.solve(self.vHt_g, dens.rhot_g,
                                           charge=-dens.charge,
                                           timer=self.timer)
        self.timer.stop('Poisson')

        self.timer.start('Hartree integrate/restrict')
        e_coulomb = 0.5 * self.finegd.integrate(self.vHt_g, dens.rhot_g,
                                                global_integral=False)

        for vt_g in self.vt_sg:
            vt_g += self.vHt_g

        self.timer.stop('Hartree integrate/restrict')
        energies = np.array([e_coulomb, e_zero, e_external, e_xc])
        return energies

    def calculate_kinetic_energy(self, density):
        # XXX new timer item for kinetic energy?
        self.timer.start('Hartree integrate/restrict')
        self.restrict_and_collect(self.vt_sg, self.vt_sG)

        e_kinetic = 0.0
        s = 0
        for vt_G, nt_G in zip(self.vt_sG, density.nt_sG):
            if self.ref_vt_sG is not None:
                vt_G += self.ref_vt_sG[s]

            if s < self.nspins:
                e_kinetic -= self.gd.integrate(vt_G, nt_G - density.nct_G,
                                               global_integral=False)
            else:
                e_kinetic -= self.gd.integrate(vt_G, nt_G,
                                               global_integral=False)
            s += 1
        self.timer.stop('Hartree integrate/restrict')
        return e_kinetic

    def calculate_atomic_hamiltonians(self, dens):
        def getshape(a):
            return sum(2 * l + 1 for l, _ in enumerate(self.setups[a].ghat_l)),
        W_aL = ArrayDict(self.atomdist.aux_partition, getshape, float)
        if self.vext:
            if self.vext.get_name() != 'CDFTPotential':
                vext_g = self.vext.get_potential(self.finegd)
                dens.ghat.integrate(self.vHt_g + vext_g, W_aL)
            else:
                dens.ghat.integrate(self.vHt_g, W_aL)
        else:
            dens.ghat.integrate(self.vHt_g, W_aL)

        return self.atomdist.to_work(self.atomdist.from_aux(W_aL))

    def calculate_forces2(self, dens, ghat_aLv, nct_av, vbar_av):
        if self.nspins == 2:
            vt_G = self.vt_sG.mean(0)
        else:
            vt_G = self.vt_sG[0]

        self.vbar.derivative(dens.nt_g, vbar_av)
        if self.vext:
            if self.vext.get_name() == 'CDFTPotential':
                # CDFT force added in calculate_forces
                dens.ghat.derivative(self.vHt_g, ghat_aLv)
            else:
                vext_g = self.vext.get_potential(self.finegd)
                dens.ghat.derivative(self.vHt_g + vext_g, ghat_aLv)
        else:
            dens.ghat.derivative(self.vHt_g, ghat_aLv)
        dens.nct.derivative(vt_G, nct_av)

    def get_electrostatic_potential(self, dens):
        self.update(dens)

        v_g = self.finegd.collect(self.vHt_g, broadcast=True)
        v_g = self.finegd.zero_pad(v_g)
        if hasattr(self.poisson, 'correction'):
            assert self.poisson.c == 2
            v_g[:, :, 0] = self.poisson.correction
        return v_g



import sys
from optparse import OptionParser

import numpy as np
from scipy.optimize import bisect

from gpaw.atom.atompaw import AtomPAW
from gpaw.atom.basis import rsplit_by_norm, get_gaussianlike_basis_function
from gpaw.atom.radialgd import EquidistantRadialGridDescriptor
from gpaw.basis_data import BasisFunction, Basis
from gpaw.hgh import (setups as hgh_setups, sc_setups as hgh_sc_setups,
                      HGHSetupData)


def atompaw(setup, f_ln, rcut, **kwargs):
    return AtomPAW(setup.symbol,
                   [f_ln],
                   rcut=rcut,
                   setups={setup.symbol : setup},
                   **kwargs)


def get_orbitals_by_energy_shift(opts, setup, **kwargs):
    h = opts.grid
    try:
        f_ln = setup.f_ln
    except AttributeError:
        f_ln = []
        for n, l, f in zip(setup.n_j, setup.l_j, setup.f_j):
            if n < 0:
                continue
            if l == len(f_ln):
                f_ln.append([])
            if f > 0 and n > 0:
                f_ln[l].append(f)

    def calculate(rcut, h=h, txt='-'):
        return atompaw(setup, f_ln, rcut, h=h, txt=txt, **kwargs)

    def get_orbital_energy(l0, n0, rcut):
        calc = calculate(rcut, 0.15, txt=None)
        for l, n, f, eps, psit_G in calc.state_iter():
            if l == l0 and n + 1 == n0:  # XXX
                return eps
        raise ValueError('No such valence state: l=%d, n=%d' % (l0, n0))

    calc0 = calculate(2.0, 0.2, txt=None)  # XXX

    def valence_states():
        for l, n, f, eps, psit_G in calc0.state_iter():
            yield l, n + 1  # XXX

    bf_j = []

    for i, (l, n) in enumerate(valence_states()):
        e0 = get_orbital_energy(l, n, 15.0) * 27.211  # 15Ang == infinity
        print('e0', e0)

        def obj(rcut):
            eps = get_orbital_energy(l, n, rcut) * 27.211
            de = eps - opts.energy_shift - e0
            # print rcut, eps, de
            return de

        # Not exactly efficient...
        rcut = bisect(obj, 1.0, 15.0, xtol=0.1)
        calc = calculate(h=h, rcut=rcut, txt=None)
        bfs = calc.extract_basis_functions()
        bf_j.append(bfs.bf_j[i])

    d = bfs.d
    ng = max([bf.ng for bf in bf_j])
    rgd = EquidistantRadialGridDescriptor(d, ng)
    basis = Basis(setup.symbol, 'strange', readxml=False, rgd=rgd)
    basis.bf_j = bf_j
    return basis
    # for (l, n), cutoff in zip(valence_states(), cutoffs):
    #     calculate(

    # return
    # calc = calculate(rcut)
    bfs = calc.extract_basis_functions(basis_name=opts.name)
    ldict = dict([(bf.l, bf) for bf in bfs.bf_j])

    rgd = bfs.get_grid_descriptor()

    def get_rsplit(bf, splitnorm):
        if opts.s_approaches_zero and bf.l == 0:
            l = 1 # create a function phi(r) = A * r + O(r^2)
        else:
            l = bf.l
        return rsplit_by_norm(rgd,
                              l,
                              bf.phit_g * rgd.r_g,
                              splitnorm**2,
                              sys.stdout)

    splitvalence_bfs = []
    for splitnorm in opts.splitnorm:
        splitnorm = float(splitnorm)
        for orbital_bf in bfs.bf_j:
            rsplit, normsqr, phit_g = get_rsplit(orbital_bf, splitnorm)
            phit_g[1:] /= rgd.r_g[1:]
            gcut = rgd.ceil(rsplit)
            #tailnorm = np.dot(rgd.dr_g[gcut:],
            #                  (rgd.r_g[gcut:] * orbital_bf.phit_g[gcut:])**2)**0.5
            #print 'tailnorm', tailnorm
            dphit_g = orbital_bf.phit_g[:gcut+1] - phit_g[:gcut+1]
            bf = BasisFunction(l=orbital_bf.l,
                               rc=rgd.r_g[gcut],
                               phit_g=dphit_g,
                               type='%s split-valence' % 'spd'[orbital_bf.l])
            splitvalence_bfs.append(bf)
    bfs.bf_j.extend(splitvalence_bfs)

    #rpol = None
    for l in range(3):
        if l not in ldict:
            lpol = l
            source_bf = ldict[lpol - 1]
            break
    else:
        raise NotImplementedError('f-type polarization not implemented')

    for splitnorm in opts.polarization:
        splitnorm = float(splitnorm)
        rchar, normsqr, phit_g = get_rsplit(source_bf, splitnorm)
        gcut = rgd.ceil(3.5 * rchar)
        rcut = rgd.r_g[gcut]
        phit_g = get_gaussianlike_basis_function(rgd, lpol, rchar, gcut)
        N = len(phit_g)
        x = np.dot(rgd.dr_g[:N], (phit_g * rgd.r_g[:N])**2)**0.5
        print('x', x)
        bf = BasisFunction(None, lpol,
                           rc=rcut,
                           phit_g=phit_g,
                           type='%s polarization' % 'spd'[lpol])
        bf.phit_g
        bfs.bf_j.append(bf)
    bfs.write_xml()


def build_parser():
    usage = '%prog [OPTION] [SYMBOL...]'

    description = 'generate basis sets from existing setups.'

    p = OptionParser(usage=usage, description=description)
    p.add_option('--grid', default=0.05, type=float, metavar='DR',
                 help='grid spacing in atomic calculation')
    #p.add_option('--rcut', default=10.0, type=float,
    #             help='radial cutoff for atomic calculation')
    p.add_option('-f', '--file', action='store_true',
                 help='load setups directly from files')
    p.add_option('-E', '--energy-shift', type=float, default=0.1,
                 help='use given energy shift to determine cutoff')
    p.add_option('-n', '--name', default='apaw',
                 help='name of basis set')
    #p.add_option('--splitnorm', action='append', default=[], metavar='NORM',
    #             help='add split-valence basis functions at this'
    #             ' tail norm.  Multiple options accumulate')
    #p.add_option('--polarization', action='append', default=[],
    #             metavar='NORM',
    #             help='add polarization function with characteristic radius'
    #             ' determined by tail norm.  Multiple options accumulate')
    #p.add_option('--s-approaches-zero', action='store_true',
    #             help='force s-type split-valence functions to 0 at origin')
    #p.add_option('-t', '--type',
    #             help='string describing extra basis functions')

    return p


def main():
    p = build_parser()
    opts, args = p.parse_args()

    if opts.file:
        for fname in args:
            tokens = fname.split('.')
            symbol = tokens[0]
            xc = tokens[1]
            name = tokens[2]
            if tokens[-1] == 'gz':
                import gzip
                fopen = gzip.open
            else:
                fopen = open
            source = fopen(fname).read()

            from gpaw.setup_data import SetupData

            s = SetupData(symbol, xc, name, readxml=False)
            s.read_xml(source=source)
            basis = get_orbitals_by_energy_shift(opts, s, xc=xc)
            basis.write_xml()
    else:
        for arg in args:
            setup = hgh_setups.get(arg)
            if setup is None:
                setup = hgh_sc_setups.get(arg.split('.')[0])
            if setup is None:
                raise ValueError('Unknown setup %s' % arg)
            print(setup)
            basis = get_orbitals_by_energy_shift(opts, HGHSetupData(setup))
            basis.write_xml()
            #generate_basis(opts, HGHSetupData(setup))



from ase import Atoms
from gpaw import GPAW, LCAO
from gpaw.test import equal
from gpaw.poisson import FDPoissonSolver

a = 6.0
b = a / 2
mol = Atoms('OHH',
            [(b, b, 0.1219 + b),
             (b, 0.7633 + b, -0.4876 + b),
             (b, -0.7633 + b, -0.4876 + b)],
            pbc=False, cell=[a, a, a])
calc = GPAW(gpts=(32, 32, 32), nbands=4, mode='lcao', poissonsolver=FDPoissonSolver())
mol.set_calculator(calc)
e = mol.get_potential_energy()
niter = calc.get_number_of_iterations()

equal(e, -10.271, 2e-3)
equal(niter, 8, 1)

# Check that complex wave functions are allowed with gamma point calculations
calc = GPAW(gpts=(32, 32, 32), nbands=4, mode=LCAO(force_complex_dtype=True), poissonsolver=FDPoissonSolver())
mol.set_calculator(calc)
ec = mol.get_potential_energy()
equal(e, ec, 1e-5)



# Test that the atomic corrections of LCAO work correctly,
# by verifying that the different implementations yield the same numbers.
#
# For example the corrections P^* dH P to the Hamiltonian.
#
# This is done by invoking GPAW once for each type of calculation.

from ase.build import molecule, bulk
from gpaw import GPAW, LCAO
from gpaw.mpi import world
from itertools import count



def test(system, **kwargs):
    corrections = ['dense', 'sparse']

    counter = count()
    energies = []
    for correction in corrections:
        parallel = {}
        if world.size >= 4:
            parallel['band'] = 2
            #if correction.name != 'dense':
            parallel['sl_auto'] = True
        calc = GPAW(mode=LCAO(atomic_correction=correction),
                    basis='sz(dzp)',
                    #spinpol=True,
                    parallel=parallel,
                    txt='gpaw.{}.txt'.format(next(counter)),
                    h=0.35, **kwargs)

        def stopcalc():
            calc.scf.converged = True

        calc.attach(stopcalc, 2)
        system.set_calculator(calc)
        energy = system.get_potential_energy()
        energies.append(energy)
        if calc.world.rank == 0:
            print('e', energy)

    master = calc.wfs.world.rank == 0
    if master:
        print('energies', energies)

    eref = energies[0]
    errs = []
    for energy, c in zip(energies, corrections):
        err = abs(energy - eref)
        errs.append(err)
        if master:
            print('err=%e :: name=%s' % (err, correction))

    maxerr = max(errs)
    assert maxerr < 1e-11, maxerr

# Use a cell large enough that some overlaps are zero.
# Thus the matrices will have at least some sparsity.
system = molecule('CH3CH2OH')
system.center(vacuum=3.0)
system.pbc = (0, 1, 1)
system = system.repeat((1, 1, 2))
system.rattle(stdev=0.05)

test(system)


system2 = bulk('Cu', orthorhombic=True) * (2, 1, 2)
test(system2, kpts=[2, 3, 4])



import pickle
import numpy as np
from os.path import isfile
from ase.parallel import rank, barrier
from gpaw.utilities import unpack2
from gpaw.lcao.projected_wannier import dots
from gpaw.utilities.tools import tri2full
from gpaw.lfc import NewLocalizedFunctionsCollection as LFC
from ase.units import Bohr, Hartree
from gpaw.utilities.timing import StepTimer, nulltimer

"""This module is used to calculate the electron-phonon coupling matrix,
    expressed in terms of GPAW LCAO orbitals."""


class ElectronPhononCouplingMatrix:
    """Class for calculating the electron-phonon coupling matrix, defined
    by the electron phonon interaction.

    ::

                  __                   _____
                  \     l   cc        /  h             cc
        H      =   )   M   c   c     /------   ( b  + b   ),
         el-ph    /_    ij  i   j  \/   2 W       l    l
                    l,ij                     l

    where the electron phonon coupling matrix is given by::

            l           ___
            M   = < i | \ /  V   * v  |j>
             ij          'u   eff   l

    """

    def __init__(self, atoms, indices=None, name='v', delta=0.005, nfree=2,
                 derivativemethod='tci'):
        assert nfree in [2, 4]
        self.nfree = nfree
        self.delta = delta

        if indices is None:
            indices = range(len(self.atoms))
        self.calc = atoms.get_calculator()
        self.atoms = atoms
        self.indices = np.asarray(indices)
        self.name = name
        self.p0 = self.atoms.positions.copy()

        self.derivativemethod = derivativemethod
        if derivativemethod == 'grid':
            self.get_dP_aMix = get_grid_dP_aMix
        elif derivativemethod == 'grid2':
            self.get_dP_aMix = get_grid2_dP_aMix
        elif derivativemethod == 'tci':
            self.get_dP_aMix = get_tci_dP_aMix
        else:
            raise ValueError('derivativemethod must be grid, grid2, or tci')

    def run(self):
        if not isfile(self.name + '.eq.pckl'):

            self.calc.calculate(self.atoms)
            Vt_G = self.calc.get_effective_potential(pad=False)
            Vt_G = self.calc.wfs.gd.collect(Vt_G, broadcast=True) / Hartree
            dH_asp = self.calc.hamiltonian.dH_asp
            setups = self.calc.wfs.setups
            nspins = self.calc.wfs.nspins
            gd_comm = self.calc.wfs.gd.comm


            alldH_asp = {}
            for a, setup in enumerate(setups):
                ni = setup.ni
                nii = ni * (ni + 1) // 2
                tmpdH_sp = np.zeros((nspins, nii))
                if a in dH_asp:
                    tmpdH_sp[:] = dH_asp[a]
                gd_comm.sum(tmpdH_sp)
                alldH_asp[a] = tmpdH_sp

            forces = self.atoms.get_forces()
            self.calc.write('eq.gpw')

            barrier()
            if rank == 0:
                vd = open(self.name + '.eq.pckl', 'wb')
                fd = open('vib.eq.pckl','wb')
                pickle.dump((Vt_G, alldH_asp), vd,2)
                pickle.dump(forces,fd)
                vd.close()
                fd.close()
            barrier()

        p = self.atoms.positions.copy()
        for a in self.indices:
            for j in range(3):
                for sign in [-1,1]:
                    for ndis in range(1,self.nfree/2+1):
                        name = '.%d%s%s.pckl' % (a, 'xyz'[j], ndis * ' +-'[sign])
                        if isfile(self.name + name):
                            continue
                        self.atoms.positions[a, j] = p[a, j] + sign * ndis * self.delta
                        self.calc.calculate(self.atoms)
                        Vt_G = self.calc.get_effective_potential(pad=False)
                        Vt_G =self.calc.wfs.gd.collect(Vt_G, broadcast=True) / Hartree
                        dH_asp = self.calc.hamiltonian.dH_asp

                        alldH_asp = {}
                        for a2, setup in enumerate(setups):
                            ni = setup.ni
                            nii = ni * (ni + 1) // 2
                            tmpdH_sp = np.zeros((nspins, nii))
                            if a2 in dH_asp:
                                tmpdH_sp[:] = dH_asp[a2]
                            gd_comm.sum(tmpdH_sp)
                            alldH_asp[a2] = tmpdH_sp

                        forces = self.atoms.get_forces()
                        barrier()
                        if rank == 0:
                            vd = open(self.name + name , 'wb')
                            fd = open('vib' + name, 'wb')
                            pickle.dump((Vt_G, alldH_asp), vd)
                            pickle.dump(forces, fd)
                            vd.close()
                            fd.close()
                        barrier()
                        self.atoms.positions[a, j] = p[a, j]
        self.atoms.set_positions(p)

    def get_gradient(self):
        """Calculates gradient"""
        nx = len(self.indices) * 3
        veqt_G, dHeq_asp = pickle.load(open(self.name + '.eq.pckl', 'rb'))
        gpts = veqt_G.shape
        dvt_Gx = np.zeros(gpts + (nx, ))
        ddH_aspx = {}
        for a, dH_sp in dHeq_asp.items():
            ddH_aspx[a] = np.empty(dH_sp.shape + (nx,))

        x = 0
        for a in self.indices:
            for i in range(3):
                name = '%s.%d%s' % (self.name,a,'xyz'[i])
                vtm_G, dHm_asp = pickle.load(open(name + '-.pckl', 'rb'))
                vtp_G, dHp_asp = pickle.load(open(name + '+.pckl', 'rb'))


                if self.nfree==4:
                    vtmm_G, dHmm_asp = pickle.load(open(name+'--.pckl', 'rb'))
                    vtpp_G, dHpp_asp = pickle.load(open(name+'++.pckl', 'rb'))
                    dvtdx_G = (-vtpp_G+8.0*vtp_G
                                -8.0*vtm_G+vtmm_G)/(12.0*self.delta/Bohr)
                    dvt_Gx[:,:,:,x] = dvtdx_G
                    for atom, ddH_spx in ddH_aspx.items():
                        ddH_aspx[atom][:,:,x] =(-dHpp_asp[atom]
                                                +8.0 * dHp_asp[atom]
                                                -8.0 * dHm_asp[atom]
                                                +dHmm_asp[atom]) / (12 * self.delta / Bohr)
                else: # nfree = 2
                    dvtdx_G = (vtp_G - vtm_G) / (2 * self.delta / Bohr)
                    dvt_Gx[..., x] = dvtdx_G
                    for atom, ddH_spx in ddH_aspx.items():
                        ddH_aspx[atom][:, :, x] = (dHp_asp[atom]
                                    -dHm_asp[atom]) / (2 * self.delta / Bohr)
                x+=1
        return dvt_Gx, ddH_aspx

    def get_M(self, modes, log=None, q=0):
        """Calculate el-ph coupling matrix for given modes(s).

        XXX:
        kwarg "q=0" is an ugly hack for k-points.
        There shuold be loops over q!

        Note that modes must be given as a dictionary with mode
        frequencies in eV and corresponding mode vectors in units
        of 1/sqrt(amu), where amu = 1.6605402e-27 Kg is an atomic mass unit.
        In short frequencies and mode vectors must be given in ase units.

        ::

                  d                   d  ~
            < w | -- v | w' > = < w | -- v | w'>
                  dP                  dP

                               _
                              \        ~a     d   .       ~a
                            +  ) < w | p  >   -- /_\H   < p | w' >
                              /_        i     dP     ij    j
                              a,ij

                               _
                              \        d  ~a     .        ~a
                            +  ) < w | -- p  >  /_\H    < p | w' >
                              /_       dP  i        ij     j
                              a,ij

                               _
                              \        ~a     .        d  ~a
                            +  ) < w | p  >  /_\H    < -- p  | w' >
                              /_        i        ij    dP  j
                              a,ij

        """
        if log is None:
            timer = nulltimer
        elif log == '-':
            timer = StepTimer(name='EPCM')
        else:
            timer = StepTimer(name='EPCM', out=open(log, 'w'))

        modes1 = modes.copy()
        #convert to atomic units
        amu = 1.6605402e-27 # atomic unit mass [Kg]
        me = 9.1093897e-31  # electron mass    [Kg]
        modes = {}
        for k in modes1.keys():
            modes[k / Hartree] = modes1[k] / np.sqrt(amu / me)

        dvt_Gx, ddH_aspx = self.get_gradient()

        from gpaw import restart
        atoms, calc = restart('eq.gpw', txt=None)
        if calc.wfs.S_qMM is None:
            calc.initialize(atoms)
            calc.initialize_positions(atoms)

        wfs = calc.wfs
        nao = wfs.setups.nao
        bfs = wfs.basis_functions
        dtype = wfs.dtype
        spin = 0 # XXX

        M_lii = {}
        timer.write_now('Starting gradient of pseudo part')
        for f, mode in modes.items():
            mo = []
            M_ii = np.zeros((nao, nao), dtype)
            for a in self.indices:
                mo.append(mode[a])
            mode = np.asarray(mo).flatten()
            dvtdP_G = np.dot(dvt_Gx, mode)
            bfs.calculate_potential_matrix(dvtdP_G, M_ii, q=q)
            tri2full(M_ii, 'L')
            M_lii[f] = M_ii
        timer.write_now('Finished gradient of pseudo part')

        P_aqMi = calc.wfs.P_aqMi
        # Add the term
        #  _
        # \        ~a     d   .       ~a
        #  ) < w | p  >   -- /_\H   < p | w' >
        # /_        i     dP     ij    j
        # a,ij

        Ma_lii = {}
        for f, mode in modes.items():
            Ma_lii[f] = np.zeros_like(M_lii.values()[0])

        timer.write_now('Starting gradient of dH^a part')
        for f, mode in modes.items():
            mo = []
            for a in self.indices:
                mo.append(mode[a])
            mode = np.asarray(mo).flatten()

            for a, ddH_spx in ddH_aspx.items():
                ddHdP_sp = np.dot(ddH_spx, mode)
                ddHdP_ii = unpack2(ddHdP_sp[spin])
                Ma_lii[f] += dots(P_aqMi[a][q], ddHdP_ii, P_aqMi[a][q].T)
        timer.write_now('Finished gradient of dH^a part')

        timer.write_now('Starting gradient of projectors part')
        dP_aMix = self.get_dP_aMix(calc.spos_ac, wfs, q, timer)
        timer.write_now('Finished gradient of projectors part')

        dH_asp = pickle.load(open('v.eq.pckl', 'rb'))[1]

        Mb_lii = {}
        for f, mode in modes.items():
            Mb_lii[f] = np.zeros_like(M_lii.values()[0])

        for f, mode in modes.items():
            for a, dP_Mix in dP_aMix.items():
                dPdP_Mi = np.dot(dP_Mix, mode[a])
                dH_ii = unpack2(dH_asp[a][spin])
                dPdP_MM = dots(dPdP_Mi, dH_ii, P_aqMi[a][q].T)
                Mb_lii[f] -= dPdP_MM + dPdP_MM.T
                # XXX The minus sign here is quite subtle.
                # It is related to how the derivative of projector
                # functions in GPAW is calculated.
                # More thorough explanations, anyone...?

        # Units of M_lii are Hartree/(Bohr * sqrt(m_e))
        for mode in M_lii.keys():
            M_lii[mode] += Ma_lii[mode] + Mb_lii[mode]

        # conversion to eV. The prefactor 1 / sqrt(hb^2 / 2 * hb * f)
        # has units Bohr * sqrt(me)
        M_lii_1 = M_lii.copy()
        M_lii = {}

        for f in M_lii_1.keys():
            M_lii[f * Hartree] =  M_lii_1[f] * Hartree / np.sqrt(2 * f)

        return M_lii



#####################################################
# XXX grid and grid 2 sometimes gives random numbers,
# XXX sometimes even nan!
#####################################################

def get_grid_dP_aMix(spos_ac, wfs, q, timer=nulltimer): # XXXXXX q
    nao = wfs.setups.nao
    C_MM = np.identity(nao, dtype=wfs.dtype)
    # XXX In the future use the New Two-Center integrals
    # to evaluate this
    dP_aMix = {}
    for a, setup in enumerate(wfs.setups):
        ni = 0
        dP_Mix = np.zeros((nao, setup.ni, 3))
        pt = LFC(wfs.gd, [setup.pt_j],
                 wfs.kd.comm, dtype=wfs.dtype, forces=True)
        spos1_ac = [spos_ac[a]]
        pt.set_k_points(wfs.ibzk_qc)
        pt.set_positions(spos1_ac)
        for b, setup_b in enumerate(wfs.setups):
            nao = setup_b.nao
            phi_MG = wfs.gd.zeros(nao, wfs.dtype)
            phi_MG = wfs.gd.collect(phi_MG, broadcast=False)
            wfs.basis_functions.lcao_to_grid(C_MM[ni:ni + nao], phi_MG, q)
            dP_bMix = pt.dict(len(phi_MG), derivative=True)
            pt.derivative(phi_MG, dP_bMix, q=q)
            dP_Mix[ni:ni + nao] = dP_bMix[0]
            ni += nao
            timer.write_now('projector grad. doing atoms (%s, %s) ' %
                            (a, b))

        dP_aMix[a] = dP_Mix
    return dP_aMix


def get_grid2_dP_aMix(spos_ac, wfs, q, *args, **kwargs): # XXXXXX q
    nao = wfs.setups.nao
    C_MM = np.identity(nao, dtype=wfs.dtype)
    bfs = wfs.basis_functions
    phi_MG = wfs.gd.zeros(nao, wfs.dtype)
    bfs.lcao_to_grid(C_MM, phi_MG, q)
    setups = wfs.setups
    pt = LFC(wfs.gd, [setup.pt_j for setup in setups],
             wfs.kd.comm, dtype=wfs.dtype, forces=True)
    pt.set_k_points(wfs.ibzk_qc)
    pt.set_positions(spos_ac)
    dP_aMix = pt.dict(len(phi_MG), derivative=True)
    pt.derivative(phi_MG, dP_aMix, q=q)
    return dP_aMix


def get_tci_dP_aMix(spos_ac, wfs, q, *args, **kwargs):
    # container for spline expansions of basis function-projector pairs
    # (note to self: remember to conjugate/negate because of that)
    from gpaw.lcao.overlap import ManySiteDictionaryWrapper,\
        TwoCenterIntegralCalculator, NewTwoCenterIntegrals

    if not isinstance(wfs.tci, NewTwoCenterIntegrals):
        raise RuntimeError('Please remember --gpaw=usenewtci=True')

    dP_aqxMi = {}
    nao = wfs.setups.nao
    nq = len(wfs.ibzk_qc)
    for a, setup in enumerate(wfs.setups):
        dP_aqxMi[a] = np.zeros((nq, 3, nao, setup.ni), wfs.dtype)

    calc = TwoCenterIntegralCalculator(wfs.ibzk_qc, derivative=True)
    expansions = ManySiteDictionaryWrapper(wfs.tci.P_expansions, dP_aqxMi)
    calc.calculate(wfs.tci.atompairs, [expansions], [dP_aqxMi])

    dP_aMix = {}
    for a in dP_aqxMi:
        dP_aMix[a] = dP_aqxMi[a].transpose(0, 2, 3, 1).copy()[q] # XXX q
    return dP_aMix



from __future__ import print_function
from ase import Atoms
from gpaw import GPAW
from gpaw.test import equal

from gpaw.utilities.kspot import AllElectronPotential
if 1:
    be = Atoms(symbols='Be',positions=[(0,0,0)])
    be.center(vacuum=5)
    calc = GPAW(gpts=(64,64,64), xc='LDA', nbands=1) #0.1 required for accuracy
    be.set_calculator(calc)
    e = be.get_potential_energy()
    niter = calc.get_number_of_iterations()
    #calc.write("be.gpw")

    energy_tolerance = 0.001
    niter_tolerance = 0
    equal(e, 0.00246471, energy_tolerance)

#be, calc = restart("be.gpw")
AllElectronPotential(calc).write_spherical_ks_potentials('bepot.txt')
f = open('bepot.txt')
lines = f.readlines()
f.close()
mmax = 0
for l in lines:
    mmax = max(abs(eval(l.split(' ')[3])), mmax)

print("Max error: ", mmax)
assert mmax<0.009
